//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Utility.h"
#include "XMLUtilities.h"
#include "MachineMod.h"
#pragma package(smart_init)
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
__fastcall TMachineDB::TMachineDB(TComponent* Owner)
	: TDataModule(Owner)
{
	GetDocFile(MachineDesired->Name);
	GetDocFile(MachineActual->Name);
	GetDocFile(MachineFlags->Name);
}

//===========================================================================
// Handlers for MachineProperties DB and XMLDoc
//---------------------------------------------------------------------------
bool __fastcall TMachineDB::OpenMachineProperties()
{/// Try to read in the machine properties table
	bool ret = true;
	AnsiString fn = Util->DBPath() + MachineProperties->Name+".xml";
	try{
		MachineProperties->LoadFromFile(fn);
	}
	catch(...){/// Failed to open file
		MachineProperties->Close();
		ret = false;
	}
	return ret;
}
//---------------------------------------------------------------------------
bool __fastcall TMachineDB::ChangeProperty(AnsiString field,AnsiString value)
{/// Edit the property in the db, post change and merge change log
	bool ret = true;
	try{
		if(OpenMachineProperties()){																							/// Try to update the db with this info
			MachineProperties->First();
			MachineProperties->Edit();
			MachineProperties->FieldByName(field)->AsString = value;
			MachineProperties->Post();
			MachineProperties->MergeChangeLog();
			SavePropertiesTable();
		}
	}
	catch(...){
		ret = false;
	}
	return ret;
}
//---------------------------------------------------------------------------
bool __fastcall TMachineDB::CreatePropertiesXMLRPC()
{/// Create an XML Document based on the values in the MachineProperties table
	bool ret = true;
	TStringList * xml = new TStringList();
	AnsiString s;
	try{/// Go node by node in the MachinePropXML Doc and create the needed nodes
		if(OpenMachineProperties()){																								/// Open the DB with the entries
			MachineProperties->First();
			xml->Add("<MachineProperties>");
			xml->Add(XMLUtil->XMLRPCStartStruct());
			s = MachineProperties->FieldByName("GUID")->AsString;
			xml->Add(XMLUtil->XMLRPCStructMember("GUID",s));
			s = MachineProperties->FieldByName("SerialNumber")->AsString;
			xml->Add(XMLUtil->XMLRPCStructMember("SerialNumber",s));
			s = MachineProperties->FieldByName("Name")->AsString;
			xml->Add(XMLUtil->XMLRPCStructMember("Name",s));
			s = MachineProperties->FieldByName("COMPort")->AsString;
			xml->Add(XMLUtil->XMLRPCStructMember("COMPort",s));
			xml->Add(XMLUtil->XMLRPCEndStruct());
			xml->Add("</MachineProperties>");
			MachinePropXML->Active = true;
			MachinePropXML->XML = xml;
			AnsiString fn;
			fn = Util->XMLPath() + MachinePropXML->Name+".xml";
			MachinePropXML->Active = true;
			MachinePropXML->SaveToFile(fn);
			MachinePropXML->FileName = fn;
		}

	}
	catch(...){
		ret = false;
	}
	delete xml;
	return ret;
}
//---------------------------------------------------------------------------
void __fastcall TMachineDB::SavePropertiesTable()
{/// Save the current machine properties table
	AnsiString fn;
	fn = Util->DBPath() + MachineProperties->Name+".xml";
	MachineProperties->SaveToFile(fn,dfXMLUTF8);
}
//===========================================================================
/// Error code handling
//---------------------------------------------------------------------------
bool __fastcall TMachineDB::AddError(int errCode, AnsiString errstrn,int severity,AnsiString action)
{///Add a record to the ErrorDB
	bool ret = true;
	try{
		if(!MachineErrorTable->Active){
		 MachineErrorTable->Active=true;
		}
		MachineErrorTable->Edit();
		MachineErrorTable->Append();
		MachineErrorTable->FieldByName("ErrorCode")->AsInteger = errCode;
		MachineErrorTable->FieldByName("ErrorString")->AsString = errstrn;
		MachineErrorTable->FieldByName("ErrorDateTime")->AsString = DateTimeToStr(Now());
		MachineErrorTable->FieldByName("ErrorSeverity")->AsInteger = severity;
		MachineErrorTable->FieldByName("Action")->AsString = action;
		MachineErrorTable->Post();
		MachineErrorTable->MergeChangeLog();
	}
	catch(...){
		ret = false;
	}
	return ret;
}
//---------------------------------------------------------------------------
void __fastcall TMachineDB::SaveErrorTable()
{/// Save the current status of the error table
	AnsiString fn;
	fn = Util->DBPath() + MachineErrorTable->Name+".xml";
	MachineErrorTable->SaveToFile(fn,dfXMLUTF8);
}
//---------------------------------------------------------------------------
bool __fastcall TMachineDB::CreateErrorXMLRPC()
{/// Create a table with all of the logged errors in it
	bool ret = true;
	TStringList * xml = new TStringList();
	try{/// Go node by node in the MachinePropXML Doc and create the needed nodes
		if(!MachineErrorTable->Active){
		 MachineErrorTable->Active=true;
		}
		xml->Add("<MachineErrorLog>");
		xml->Add(XMLUtil->XMLRPCStartStruct());
		while(!MachineErrorTable->Eof){																						/// Loop over the table and add the values
			int 			 i 	= MachineErrorTable->FieldByName("ErrorCode")->AsInteger;
			xml->Add(XMLUtil->XMLRPCStructMember("ErrorString",i));
			AnsiString s 	= MachineErrorTable->FieldByName("ErrorString")->AsString;
			xml->Add(XMLUtil->XMLRPCStructMember("ErrorString",s));
			s 						= MachineErrorTable->FieldByName("ErrorDateTime")->AsString;
			xml->Add(XMLUtil->XMLRPCStructMember("ErrorDateTime",s));
			i							= MachineErrorTable->FieldByName("ErrorSeverity")->AsInteger;
			xml->Add(XMLUtil->XMLRPCStructMember("ErrorSeverity",i));
			s							= MachineErrorTable->FieldByName("Action")->AsString;
			xml->Add(XMLUtil->XMLRPCStructMember("ErrorAction",s));
			MachineErrorTable->Next();
		}
		xml->Add(XMLUtil->XMLRPCEndStruct());
		xml->Add("<\MachineErrorLog>");
		ErrorXML->Active = true;
		ErrorXML->XML = xml;
		AnsiString fn;
		fn = Util->XMLPath() + ErrorXML->Name+".xml";
		ErrorXML->Active = true;
		ErrorXML->SaveToFile(fn);
		ErrorXML->FileName = fn;
	}
	catch(...){
		ret = false;
	}
	delete xml;
	return ret;
}
//===========================================================================
/// XMLDoc handlers for the machine
//---------------------------------------------------------------------------
TXMLDocument *TMachineDB::GetDocument(AnsiString doc)
{/// Return the pointer to the document named doc
	TXMLDocument * xml = NULL;
	if(doc == MachineDesired->Name){
		xml = MachineDesired;
	}
	if(doc == MachineActual->Name){
		xml = MachineActual;
	}
	if(doc == MachineFlags->Name){
		xml = MachineFlags;
	}
	return xml;
}
//---------------------------------------------------------------------------
_di_IXMLNode 	__fastcall TMachineDB::GetRPCNode(TXMLDocument *xml,AnsiString name)
{/// Return the node called name in document xmldoc
	_di_IXMLNode docnode	= NULL;										/// Holds the doc's root node
	_di_IXMLNode node 		= NULL;										/// Will hold the node with the value
	_di_IXMLNode structure 	= NULL;                 /// Holds the structure node- children of this are member nodes
	_di_IXMLNode member 	= NULL;										/// Holds the member node
	_di_IXMLNode namenode = NULL;										/// Holds the name node. Its child is the node with the value text
	_di_IXMLNode value 		= NULL;										/// Holds the value node. Its child is the node with the value text
	_di_IXMLNodeList nodelist;											/// Nodelist for the xml or structure nodes
	docnode = xml->DocumentElement;									/// Get the root node
	if(docnode->ChildNodes->First() == NULL){
		return NULL;                             			/// No nodes! Bail out
	}
	nodelist = docnode->ChildNodes;                 /// Get the root node child nodes
	structure = nodelist->Nodes[0];     						/// This will be the <struct> node
	if(structure->ChildNodes->First() == NULL){
		return NULL;                                  /// No child nodes! Bail out
	}
	nodelist = structure->GetChildNodes();          /// This will have the list of <member> nodes
	for(int i = 0;i < nodelist->Count;++i){					/// Loop over all of the member nodes
		member = nodelist->Nodes[i];      						/// Get the next <member> node
		AnsiString s = member->LocalName;             /// Get the local name for this node
		if(member == NULL){
			return NULL;																/// There isn't one! Bail out
		}
		namenode = member->ChildNodes->Nodes[0];			/// This should be the <name> node
		if(namenode == NULL ||
			!namenode->IsTextElement){
				return NULL;															/// Either this isn't a node or the <name> node is messed up
		}
		if(namenode->Text == name){										/// We've found the one we want
			if(namenode->ChildNodes->First() == NULL){
				return NULL;                              /// There isn't a value associated with this node!
			}
			value = member->ChildNodes->Nodes[1];				/// This is the value node
			AnsiString s = value->LocalName;            /// Get the local name for this node
			break;																			/// Get out of the loop
		}
	}
	if(value == NULL){
		return NULL;                              		/// There isn't a value node!
	}
	if(value->ChildNodes->First() == NULL){
		return NULL;																	/// There is not a value associated with this <value> node
	}
	node = value->ChildNodes->First();							/// This should be it!
	return node;
}
//---------------------------------------------------------------------------
_di_IXMLNode 	__fastcall TMachineDB::GetRPCNode(AnsiString doc,AnsiString name)
{/// Return the value node called 'name' in document named doc
	TXMLDocument * xml;
	xml = GetDocument(doc);													/// Get the pointer to the XMLDoc
	if(xml == NULL){
		return NULL;																	/// No document! Bail out
	}
	return GetRPCNode(xml,name);
}
//---------------------------------------------------------------------------
bool __fastcall TMachineDB::GetDocFile(AnsiString name)
{/// Load the XML file for the XMLDocs
	bool ret = true;
	TXMLDocument * xml;
	try{
		xml = GetDocument(name);										/// Get the pointer to the XMLDoc
		if(xml == NULL){
			throw 1;																 /// No document! Bail out
		}
		xml->Active = false;
		AnsiString fn = Util->XMLPath();
		fn = fn +name+".xml";
		xml->LoadFromFile(fn);
		xml->Active = true;
	}
	catch(...){
		ret = false;
	}
	return ret;
}
//---------------------------------------------------------------------------
bool __fastcall TMachineDB::ChangeValue(AnsiString doc,AnsiString what,AnsiString newvalue)
{/// Change the value of variable what in doc
	bool ret = true;
	try{
		_di_IXMLNode node = GetRPCNode(doc,what);
		if(node != NULL)
			node->Text = newvalue;
	}
	catch(...){
		ret = false;
	}
	return ret;
}
//---------------------------------------------------------------------------
AnsiString 	__fastcall TMachineDB::GetValue(AnsiString doc,AnsiString what)
{/// Get the value from doc for the what node
	AnsiString ret = "";
	try{
		_di_IXMLNode node = GetRPCNode(doc,what);
		if(node != NULL)
			ret = node->Text;
	}
	catch(...){
		ret = "";
	}
	return ret;
}
//---------------------------------------------------------------------------

