//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "MachineSocketFrm.h"
#include "ShowPortsFrm.h"
#include "Utility.h"
#include "Machine.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMSocketForm *MSocketForm;
//---------------------------------------------------------------------------
__fastcall TMSocketForm::TMSocketForm(TComponent* Owner)
	: TForm(Owner)
{
	Machine = new TMachine(this);
	Machine->MachineStatus->Parent=this;												 				///Show the machine status form
	Width = 	Machine->MachineStatus->Width;
	Height =  Machine->MachineStatus->Height + Panel1->Height+20;
	Machine->MachineStatus->Align=alClient;
	Okbtn->Left = Width/2 - Okbtn->Width/2;
	Caption = Caption + " Version: "+Util->GetCurrentVersion();
	Machine->InitializeCommPort();																			/// Try to open the port on startup
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::FormDestroy(TObject *Sender)
{
	if(Util->FormExists("PortsFrm")){																		/// If this form exists, delete it
		delete PortsFrm;
	}
	delete Machine;
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::FormShow(TObject *Sender)
{///Update the caption
	Machine->MachineStatus->Visible = true;
	Machine->MachineStatus->UpdateDisplay();
}
//---------------------------------------------------------------------------
// Menu and button items
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::PopupMenu1Popup(TObject *Sender)
{/// Show whether or not the device is connected- set menu options accordingly
	Simulationvalues1->Enabled = Machine->IsSimulation();
	if(Machine->CommunicationsEstablished()																			/// Set the connect/disconnect strings according to connection status
		|| (Machine->ConnectionState()).SubString(1,4) == "Wait"){
		Connect1->Enabled = false;
		Disconnect1->Enabled = true;
	}
	else{                                                                    		/// Not connected
		Connect1->Enabled = true;
		Disconnect1->Enabled = false;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::Exit2Click(TObject *Sender)
{/// From the tray icon
	Close();
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::OkbtnClick(TObject *Sender)
{///Pressing OK hides the app
	Hide();
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::XLAProperties1Click(TObject *Sender)
{/// Show the main form
	Show();
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::Socketproperties1Click(TObject *Sender)
{///Show the socket information
	Machine->XLSocket->Show();
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::Connect1Click(TObject *Sender)
{///Connect to XLA
	Machine->InitializeCommPort();																							/// Try to establish connection to XL
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::Disconnect1Click(TObject *Sender)
{///Disconnect from XLA
	Machine->CloseCommPort();                                                   /// Close connection to XL
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::XLAConnection1Click(TObject *Sender)
{///Show the XLA connection
	Machine->ShowCommPort();
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::XLAstatusflags1Click(TObject *Sender)
{
	Machine->ShowMachineFlags();
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::XLA1Click(TObject *Sender)
{///View the error form
	Machine->ShowErrorLog();
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::ViewmachineXMLRPC1Click(TObject *Sender)
{///Show the XMLRPC code for the machine
	Machine->ShowMachineXML();
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::Showavailableports2Click(TObject *Sender)
{/// Show what ports are available for COMPort connection
	if(Util->CanShowForm(PortsFrm)){
		return;																																		/// Form existed and is now visible
	}
	PortsFrm = new TPortsFrm(this);
	PortsFrm->Show();
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::XLAproperties2Click(TObject *Sender)
{///Edit the XLA's properties
	Machine->EditPropertiesDB();
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::Usesimulationmode1Click(TObject *Sender)
{///Toggle between simulation and non-simulation mode
	Usesimulationmode1->Checked=!Usesimulationmode1->Checked;                 /// Toggle between states
	Machine->SetSimulation(Usesimulationmode1->Checked);                      /// Let the machine know which state we're dealing with
	if(Machine->IsSimulation()){                                              /// If we are simulating the machine, make sure things are set up
		Machine->SetupForSimulation();
	}
	Machine->InitializeCommPort();																						/// Try to open the port on startup
	Machine->MachineStatus->UpdateDisplay();
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::TrayIcon1DblClick(TObject *Sender)
{/// Double click to get the machine status form
	Util->CanShowForm(this);
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::TrayIcon1MouseMove(TObject *Sender,
			TShiftState Shift, int X, int Y)
{/// Change the hint to get the status
	AnsiString s = Machine->GetSysStatus();
	TrayIcon1->Hint = s;
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::Simulationvalues1Click(TObject *Sender)
{/// Show the simulation values
	Machine->ShowSimulator();
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::Timer1Timer(TObject *Sender)
{///Determine which icon is correct
	static bool bConnected = false;
	if(Machine->CommunicationsEstablished()){
		TrayIcon1->Animate = false;
		TrayIcon1->IconIndex = 1;
		if(!bConnected){																												/// Need to show that connection has been made
			TrayIcon1->BalloonFlags = bfInfo;																			/// Jus' sayin'
			TrayIcon1->BalloonHint = Machine->ConnectionState();									/// Show the connection status
			TrayIcon1->ShowBalloonHint();																					/// Show it
			bConnected = true;																										/// Only want to fall through this once
		}
		return;
	}
	if((Machine->ConnectionState()).LowerCase() == "not connected"){
		TrayIcon1->Animate = false;
		TrayIcon1->IconIndex = 0;
		if(bConnected){																													/// Need to show that connection has been lost
			TrayIcon1->BalloonFlags = bfWarning;																	/// Warning that connection lost
			TrayIcon1->BalloonHint = Machine->ConnectionState();									/// Show the connection status
			TrayIcon1->ShowBalloonHint();																					/// Show it
			bConnected = false;																										/// Only want to fall through this once
		}
		return;
	}
	TrayIcon1->Animate = true;																								/// Default... shown while trying to connect
}
//---------------------------------------------------------------------------
void __fastcall TMSocketForm::FormCreate(TObject *Sender)
{
	ShowWindow(Application->Handle, SW_HIDE);																	/// Keep the app from the taskbar... only in the tray
}
//---------------------------------------------------------------------------


