//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "SocketTempSet.h"
#include "Utility.h"
#include "Machine.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TTempSetFrm *TempSetFrm;
//---------------------------------------------------------------------------
__fastcall TTempSetFrm::TTempSetFrm(TComponent* Owner)
  : TForm(Owner)
{
  InitialTemp=Machine->GetActualTemperatureAsInt();
  bIgnore=true;
}
//---------------------------------------------------------------------------
void TTempSetFrm::HideStatus()
{///Don't show the heating/cooling status
  StatusTxt->Visible=false;
  HeatCoolTxt->Visible=false;
}
//---------------------------------------------------------------------------
void __fastcall TTempSetFrm::FormShow(TObject *Sender)
{///Initialize the form
  CurrentTxt->Caption = "Current "+(AnsiString)InitialTemp;
  CurrentTemp=InitialTemp;
  TrackBar1->Position= (int)40-CurrentTemp;                                   ///The trackbar runs backwards... 0 is the top, 40 is the bottom
  bIgnore=true;                                                               ///Do not use the NubmerEdit handler
  Edit1->Text=(AnsiString)CurrentTemp;                                        ///Set the current temperature
	bIgnore=false;
}
//---------------------------------------------------------------------------
void __fastcall TTempSetFrm::OKBtnClick(TObject *Sender)
{
  CurrentTemp =Edit1->Text.ToInt();
  TrackBar1->Position= 40-CurrentTemp;
  Machine->TemperatureSystem->TemperatureTolerance=StrToFloat(TempTolEd->Text);
}
//---------------------------------------------------------------------------
void __fastcall TTempSetFrm::TrackBar1Change(TObject *Sender)
{
  CurrentTemp=40-TrackBar1->Position;
  bIgnore=true;
  Edit1->Text=(AnsiString)CurrentTemp;
  bIgnore=false;
  UpdateSettings();
}
//---------------------------------------------------------------------------
void TTempSetFrm::UpdateSettings()
{///Update the Heating/cooling status
    if(!StatusTxt->Visible) return;
    int i= 40- TrackBar1->Position;
    if (InitialTemp == i) HeatCoolTxt->Visible=false;
    if (InitialTemp < i)
      {///System will need to warm up
        HeatCoolTxt->Caption="Heating";
        HeatCoolTxt->Color=clRed;
        HeatCoolTxt->Visible=true;
      }
    if (InitialTemp > i)
      {///System will need to cool down
        HeatCoolTxt->Caption="Cooling";
        HeatCoolTxt->Color=clBlue;
        HeatCoolTxt->Visible=true;
      }
}
//---------------------------------------------------------------------------
void __fastcall TTempSetFrm::Edit1Enter(TObject *Sender)
{
  if(bIgnore) return;
  Ed = (TEdit *)Sender;
  NumberEdit->Initialize((TEdit *)Sender,0,40,true);
}
//---------------------------------------------------------------------------
void __fastcall TTempSetFrm::Edit1Exit(TObject *Sender)
{
  if(bIgnore) return;
  if(Ed == NULL) Ed = (TEdit *)Sender;
  if(!NumberEdit->ExitQuery()) Ed->Text=NumberEdit->InitialText;
  CurrentTemp = (Ed->Text).ToInt();
  TrackBar1->Position= 40-CurrentTemp;
}
//---------------------------------------------------------------------------
void __fastcall TTempSetFrm::Edit1KeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{
  if(bIgnore) return;
  NumberEdit->KeyDown((TEdit *)Sender,Key,Shift);
}
//---------------------------------------------------------------------------
void __fastcall TTempSetFrm::Edit1KeyPress(TObject *Sender, char &Key)
{
	if(bIgnore || Ed == NULL) return;
  Ed = (TEdit *)Sender;
	if (Key == Char(VK_RETURN))
		{
			if(Ed != NULL) Ed->OnExit(Sender);
			Key = 0;
			PostMessage(Handle, WM_NEXTDLGCTL, 0, 0);
			return;
		}
	NumberEdit->KeyPress((TEdit *)Sender,Key);
}
//---------------------------------------------------------------------------

void __fastcall TTempSetFrm::FormCreate(TObject *Sender)
{
	NumberEdit = new TNumberEdit(this);
}
//---------------------------------------------------------------------------
void __fastcall TTempSetFrm::FormDestroy(TObject *Sender)
{
	delete NumberEdit;
}
//---------------------------------------------------------------------------
void __fastcall TTempSetFrm::TempTolEdEnter(TObject *Sender)
{///Temperature tolerance enter
	if(bIgnore) return;
	Ed = (TEdit *)Sender;
	NumberEdit->Initialize((TEdit *)Sender, 0.0,2.0,false);
}
//---------------------------------------------------------------------------
void __fastcall TTempSetFrm::TempTolEdExit(TObject *Sender)
{///Temperature tolerance exit
  if(bIgnore) return;
  Ed = (TEdit *)Sender;
  if(!NumberEdit->ExitQuery()) Ed->Text=NumberEdit->InitialText;
}
//---------------------------------------------------------------------------


