//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "MachineStatusForm.h"
#include "Machine.h"
#include "RpmSet.h"
#include "TempSet.h"
#include "SetTimeFrm.h"
#include "Utility.h"
#include <math.h>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
__fastcall TMachineStatusFrm::TMachineStatusFrm(TComponent* Owner)
	: TForm(Owner)
{
	NumberEdit = new TNumberEdit(this);
	bIgnore = true;
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::FormClose(TObject *Sender,
			TCloseAction &Action)
{
	delete NumberEdit;
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::UpdateDisplay()
{///Calls the following functions
	UpdateActualValues();                                                     /// Update the actual values text boxes
	UpdateRpmLight();																													/// Determines whether the accel/decel light is on, and what color
	UpdateTempLight();																												/// Determine heating/cooling status of the light
	UpdateDesiredSettings();																									/// Update the desired values text boxes
	UpdateElapsedTime();																											/// Update the elapsed time... or time remaining
	UpdateStartStopButtons();																									/// Enable/Disable the start stop buttons depending on the machine status
	UpdateButtonStatus();																											/// Sets the other button statuses if the machine is ready to take commands
	UpdateMachineStatus();																										/// Update the system status text
	UpdateStatusBar();																												/// Update the machine status, connection status, simulation status
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::UpdateMachineStatus()
{/// Update the machine status, connection status, simulation status
	MachStatTxt->Caption = Machine->GetMachineStatus();												/// Display the machine status
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::UpdateStatusBar()
{/// Update the machine status, connection status, simulation status, etc. SB2 is lower bar, SB upper bar
	AnsiString s;
	SB2->Panels->Items[0]->Text = Machine->VacuumSystem->GetVacuumStatus();
	if(Machine->IsSimulation()){
		SB2->Panels->Items[1]->Text = "Simulation";															/// Display the simulation status as simulation
	}
	else{
		SB2->Panels->Items[1]->Text = "Live";																		/// Display the simulation status as real
	}
	SB->Panels->Items[0]->Text = Machine->ConnectionState();									/// Display the machine status
	if(Machine->XLSocket->AOSServer->Active){
		s = "Socket: running, ";
	}
	else{
		s = "Socket: not running, ";
	}
	if(Machine->XLSocket->HasConnections()){
		s= s+ "with connection";																								/// Display the socket status
	}
	else{
		s =s+ "no connections";																									/// Display the socket status
	}
	SB->Panels->Items[1]->Text = s;
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::UpdateButtonStatus()
{///Enable/disable the buttons depending on the machine status
	bool b = Machine->CanSetValues();
	UpdateTBtnStatus(b);           																						/// Updates the availability of the temperature buttons and menu item, enable set by b, with an override if the experiment is started
	UpdateRBtnStatus(b);           																						/// Updates the availability of the rotor speed buttons and menu item, enable set by b
	UpdateTimeBtnStatus(b);        																						/// Updates the availability of the time buttons and menu item, enable set by b
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::UpdateStartStopButtons()
{/// Update the start-stop buttons
	StartBtn->Enabled = Machine->CanStart();
	StopBtn->Enabled = Machine->CanStop();
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::UpdateTimeBtnStatus(bool b)
{///Updates the availability of the time buttons and menu item, enable set by b
	TimeBtn2->Enabled               =b;
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::UpdateRBtnStatus(bool b)
{///Updates the availability of the rotor speed buttons and menu item, enable set by b
	RpmBtn->Enabled                 = b;
	RpmSetTxt->Enabled              = b;
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::UpdateTBtnStatus(bool b)
{///Updates the availability of the temperature buttons and menu item
	TBtn->Enabled                   = b;
	TempSetTxt->Enabled             = b;
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::UpdateRpmLight()
{///Checks to see if the rotor speed is stable & non-zero, then updates menu display
	if(!Machine->bRotorSpinning || Machine->IsRotorSpeedStable()){																								///Rpm is zero... shut off RpmLight and bail out
		RpmLight->Visible=false;
		return;
	}
	if(Machine->RpmChange() < 0){																								/// The rotor speed has changed by more than 100 rpm since the last update
		RpmLight->Color = clBlue;  																								/// Braking-       rpm < last rpm
		RpmLight->Caption = "Decelerating";
	}
	else{
		RpmLight->Color = clRed;   																								///Accelerating-  rpm > last rpm
		RpmLight->Caption = "Accelerating";
	}
	RpmLight->Visible=true;
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::UpdateTempLight()
{///Checks for heating or cooling, updates light status
	double DifTemp=Machine->TemperatureSystem->TemperatureDifference;
	if(DifTemp==0){                                      												/// At temperature, so shut off the light and bail
		TempLight->Visible=false;
		return;
	}
	if (DifTemp > 0){
		TempLight->Color	 = clRed;  																							/// Heating
		TempLight->Caption = "Heating";
	}
	else{
		TempLight->Color=clBlue;   																								/// Cooling
		TempLight->Caption = "Cooling";
	}
	TempLight->Visible=true;                            												/// Show it
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::UpdateActualValues()
{///Update all of the information on the 'Centrifuge Status' panel
	if(!Machine->CommunicationsEstablished()){																	/// Can't know the actual values... set them to blanks
	RpmTxt->Caption  = "----";																									/// Rpm
	TempTxt->Caption = "----";           																				/// Temperature
	AcAcTxt->Caption = "----";                          												/// Acceleration
	DeAcTxt->Caption = "----";																									/// Deceleration
	return;
	}
	AnsiString s;
	if(Machine->GetActualRotorSpeed() == 0){            												/// Attempt to make the rpm text consistent
		s = "00000";
	}
	else{
		s = FormatFloat("#0000",Machine->GetActualRotorSpeed());
	}
	RpmTxt->Caption  = s;                  																			/// Formatted rpm
	TempTxt->Caption = Machine->GetActualTemperatureAsString();   							/// Display the current temperature in ##.# format, with 0
	AcAcTxt->Caption = Machine->GetActualAccelerationAsString();                /// Acceleration
	DeAcTxt->Caption = Machine->GetActualDecelerationAsString();                /// Deceleration
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::UpdateDesiredSettings()
{///Update all of the information on the 'Desired Values' panel
	bIgnore=true;
	AnsiString s;
	if(Screen->ActiveControl->Name != "RpmSetTxt"){															/// Won't try to update it if the user is entering a number
		if(Machine->VacuumSystem->bVacuumOK){																							/// The start button has been pressed, so the desired setting is OK
			int i = Machine->GetDesiredRotorSpeed()/1000;
			s=(AnsiString)i;
		}
		else{
			s = Machine->RequestedRotorSpeed;																				/// Show the user that their request is still valid
			int i = s.ToIntDef(0)/1000;
			s = (AnsiString)i;
    }
		RpmSetTxt->Text=s;
	}
	if(Screen->ActiveControl->Name != "TempSetTxt"){														/// Won't try to update it if the user is entering a number
		TempSetTxt->Text=(AnsiString)Machine->GetDesiredTemperatureAsInt();				/// This will be formatted correctly for the desired temperature
	}
	AcDeTxt->Caption = Machine->GetDesiredAccelerationAsString();
	DeDeTxt->Caption = Machine->GetDesiredDecelerationAsString();
	TimeSetTxt->Caption = Machine->GetDesiredTime();
	bIgnore=false;
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::UpdateElapsedTime()
{///Update the centrifugation time (omega-squared*time)
	if(!Machine->CommunicationsEstablished()){														/// Can't know the actual values... set them to blanks
		W2Txt->Caption="----";
	}
	else{
		W2Txt->Caption=Machine->GetW2tAsString();
	}
	if(TimeSetTxt->Caption =="Hold"){ 																		///Time to be displayed is the elapsed time
		RunTimeTxt->Caption="Elapsed:";																			///Update the elapsed time if the machine is in Hold mode, otherwise the Remaining time is displayed
	}
	else{																																	///The machine returns the remaining time
		RunTimeTxt->Caption="Remaining:" ;
	}
	TimeElapsedTxt->Caption=Machine->GetActualTimeAsString();
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::RpmBtnClick(TObject *Sender)
{///Allow the user to select a new rotor speed
	RpmSetFrm=new TRpmSetFrm(this);
	RpmSetFrm->ShowModal();
	if(RpmSetFrm->ModalResult==mrOk){
		ChangeRotorSpeed(RpmSetFrm->CurrentSpeed);
	}
	delete RpmSetFrm;
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::ChangeRotorSpeed(int rpm)
{///The user has changed the rotor speed either through the RpmSet form or the edit box
	bIgnore=true;
	AnsiString s=(AnsiString)(rpm/1000);
	RpmSetTxt->Text=s;
	bIgnore=false;
	Machine->SetRotorSpeed(rpm);
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::TBtnClick(TObject *Sender)
{///Allow the user to change the temperature
	TempSetFrm = new TTempSetFrm(this);
	TempSetFrm->ShowModal();
	if(TempSetFrm->ModalResult==mrOk){
		bool tempchanged;
		if  (TempSetTxt->Text != (AnsiString)Machine->GetDesiredTemperatureAsInt()){
			tempchanged=true;
		}
		else{
			tempchanged=false;
		}
		AnsiString s = (AnsiString)TempSetFrm->CurrentTemp;
		int i = s.ToInt();
		Machine->SetTemperature(i);
		bIgnore=true;
		s=(AnsiString)Machine->GetDesiredTemperatureAsInt();
		TempSetTxt->Text=s;
		bIgnore=false;
	}
	delete TempSetFrm;
	Machine->CheckDesiredSettings();
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::TimeBtn2Click(TObject *Sender)
{///Set the runtime using the runtime form
	TimeSetFrm = new TTimeSetFrm(this);
	TimeSetFrm->ShowModal();
	if(TimeSetFrm->ModalResult==mrOk){																					/// Save the time
			Machine->SetDesiredTime(TimeSetFrm->GetTimeAsString());
			TimeSetTxt->Caption=Machine->GetDesiredTime();
			Machine->DesiredSettingsChanged(true);
		}
	delete TimeSetFrm;
	UpdateStartStopButtons();
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::FormResize(TObject *Sender)
{
	if(!Visible) return;
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::TempSetTxtEnter(TObject *Sender)
{
	if(bIgnore) return;
	NumberEdit->Initialize((TEdit *)Sender,0,40,true);
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::TempSetTxtExit(TObject *Sender)
{
	if(bIgnore) return;
	if(!NumberEdit->ExitQuery()){
	 ((TEdit *)Sender)->Text=NumberEdit->InitialText;
	}
	AnsiString s = ((TEdit *)Sender)->Text;
	int i = s.ToInt();
	if(i != Machine->GetDesiredTemperatureAsInt()){										///Log a temperature change
		Machine->SetTemperature(i);
	}
 //	if((((TEdit *)Sender)->Text).Length() < 2) ((TEdit *)Sender)->Text=" "+((TEdit *)Sender)->Text;
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::RpmSetTxtEnter(TObject *Sender)
{
	if(bIgnore) return;
	int MaxSpeed= 60;																								//Number will be in thousands of rpm
	NumberEdit->Initialize((TEdit *)Sender,0,MaxSpeed,true);
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::RpmSetTxtExit(TObject *Sender)
{
	if(bIgnore) return;
	if(!NumberEdit->ExitQuery()){
	 ((TEdit *)Sender)->Text=NumberEdit->InitialText;
	}
	int rpm = 1000*(((TEdit *)Sender)->Text).ToInt();
	ChangeRotorSpeed(rpm);
	if((((TEdit *)Sender)->Text).Length() < 2){
	 ((TEdit *)Sender)->Text=" "+((TEdit *)Sender)->Text;
	}
}
//---------------------------------------------------------------------------

void __fastcall TMachineStatusFrm::RpmSetTxtKeyDown(TObject *Sender,
			WORD &Key, TShiftState Shift)
{
	if(bIgnore) return;
	NumberEdit->KeyDown((TEdit *)Sender,Key,Shift);
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::RpmSetTxtKeyPress(TObject *Sender,
			char &Key)
{
	if(bIgnore || (TEdit *)Sender == NULL) return;
	if (Key == Char(VK_RETURN))
		{
			if((TEdit *)Sender != NULL) ((TEdit *)Sender)->OnExit(Sender);
			Key = 0;
			SelectNext(Screen->ActiveControl, true, true);
			return;
		}
	NumberEdit->KeyPress((TEdit *)Sender,Key);
}
//---------------------------------------------------------------------------
void __fastcall TMachineStatusFrm::StartBtnClick(TObject *Sender)
{/// Start button has been pressed
	Machine->StartMachine();
	UpdateStartStopButtons();
}
//---------------------------------------------------------------------------

void __fastcall TMachineStatusFrm::StopBtnClick(TObject *Sender)
{/// Stop the machine
	Machine->StopMachine();
	UpdateStartStopButtons();
}
//---------------------------------------------------------------------------

