//---------------------------------------------------------------------------


#pragma hdrstop

#include "TemperatureSystem.h"
#include <math.h>
#include "Machine.h"

//---------------------------------------------------------------------------
__fastcall TTemperatureSystem::TTemperatureSystem(TComponent *Owner):TComponent(Owner)
{
	bTemperatureOK        = false;      ///Machine has been at the desired temperature for the time period specified by the WaitAfterTemp
	TemperatureTolerance  = 0.1;        ///What is the acceptable value of the TemperatureDifference before allowing the timing begin
}
//---------------------------------------------------------------------------
__fastcall TTemperatureSystem::~TTemperatureSystem()
{
}
//---------------------------------------------------------------------------
bool __fastcall TTemperatureSystem::TemperatureInTolerance()
{///Returns true if the temperature is within tolerance of the desired temperature
	bool b=false;
	double Tdif = fabs(TemperatureDifference);
	double Ttol = TemperatureTolerance;
	if(Tdif < Ttol) b=true;
	return b;
}
//---------------------------------------------------------------------------
void __fastcall TTemperatureSystem::SimulateTemperatureChanges(AnsiString temp)
{///Called during acceleration to simulate rotor temperature changes
	static int update = 0;
	double SetTemp=StrToFloat(temp);
	bool bAccel = false;
	if(Machine->GetMachineStatus() == "Accelerating"){
		bAccel = true;
	}
	double Ta = Machine->GetActualTemperatureAsDouble();
	TemperatureDifference=Temperature-SetTemp;
	double Tdif=TemperatureDifference;
	double tt=TemperatureTolerance;
	if(fabs(Tdif) <= tt && !bAccel){			///Within tolerance and not accelerating.
		Temperature=SetTemp;
		TemperatureDifference=0;
		Tdif=0;
	}
	if(bAccel){														///Machine is simulating acceleration, warm rotor up, but not more than 1 degree
			if(fabs(Tdif) <= 0.7 && update >= 2){///Only update every 3 rounds
				Ta = Ta+0.1;
				update = 0;
			}
			else update++;
		}
	else{
		if(update >= 2){
			if(Tdif >0)  Ta=Ta-0.1; //Cool it by 0.1 degree
			if(Tdif < 0) Ta=Ta+0.1; //Heat it by 0.1 degree
			update = 0;
		}
		else update++;
	}
	Ta=10*Ta;
	Machine->XLActual->putTemp(((AnsiString)Ta).c_str());
}
//---------------------------------------------------------------------------
bool __fastcall TTemperatureSystem::Update(AnsiString temp)
{///Update the temperature system status- returns true if update succeeds
	bool ret = true;
	Temperature = Machine->GetActualTemperatureAsDouble();
	double SetTemp=StrToFloat(temp);
	if(Machine->IsSimulation()
		&& Machine->VacuumSystem->bVacuumOK){																		///Allow simulation of temperature setting... do this only after the vacuum is below 100 microns
			SimulateTemperatureChanges(temp);
		}
	TemperatureDifference=SetTemp-Machine->GetActualTemperatureAsDouble();
	if(Machine->VacuumSystem->bVacuumOK){
		bTemperatureOK = TemperatureInTolerance();
  }
	return ret;
}
//---------------------------------------------------------------------------
#pragma package(smart_init)
