//---------------------------------------------------------------------------

#ifndef XLCalDataH
#define XLCalDataH
#include <vcl.h>
#pragma hdrstop

#define PROTECTED				TRUE	// for protecting low/hi wavelength cal points

#define	INRADLEN				6
#define	OUTRADLEN				6
#define	INCOUNTLEN			5
#define	OUTCOUNTLEN			5
#define	INLIMITLEN			5
#define	OUTLIMITLEN			5
#define	RADCALDLEN			20
#define	LOWWAVELEN			4
#define	HIWAVELEN				4
#define	LOWCOUNTLEN			5
#define	HICOUNTLEN			5
#define	LOWLIMITLEN			5
#define	HILIMITLEN			5
#define	WAVECALDLEN			20
#define	DLYCALSPDLEN		6
#define	COLLAROFFSETLEN	7

#define	DEFAULT_INRAD         "\0"
#define	DEFAULT_OUTRAD				"\0"
#define	DEFAULT_INCOUNT       "\0"
#define	DEFAULT_OUTCOUNT			"\0"
#define	DEFAULT_INLIMIT				"\0"
#define	DEFAULT_OUTLIMIT			"\0"
#define	DEFAULT_RADCALD				"\0"
#define	DEFAULT_RECALRAD			0
#define	DEFAULT_RECALWAVE			0
#define	DEFAULT_LOWWAVE				"\0"
#define	DEFAULT_HIWAVE				"\0"
#define	DEFAULT_LOWCOUNT			"\0"
#define	DEFAULT_HICOUNT				"\0"
#define	DEFAULT_LOWLIMIT			"\0"
#define	DEFAULT_HILIMIT				"\0"
#define	DEFAULT_WAVECALD 			"\0"
#define	DEFAULT_DLYCALSPD			"\0"
#define	DEFAULT_COLLAROFFSET	"\0"
#define CALDATA_SIZE	80

class XLCalData:public TComponent
{///XL calibration data
private:
  char 	inRad [INRADLEN];
  char 	outRad [OUTRADLEN];
  char 	lowWave [LOWWAVELEN];
  char 	hiWave [HIWAVELEN];
  char	protectedLowWave [LOWWAVELEN];
  char 	protectedHiWave [HIWAVELEN];
  char	collarOffset [COLLAROFFSETLEN];
  char 	inCount [INCOUNTLEN];
  char 	outCount [OUTCOUNTLEN];
  char 	inLimit [INLIMITLEN];
  char 	outLimit [OUTLIMITLEN];
  char 	radCald [RADCALDLEN];
  char 	lowCount [LOWCOUNTLEN];
  char 	hiCount [HICOUNTLEN];
  char 	lowLimit [LOWLIMITLEN];
  char 	hiLimit [HILIMITLEN];
  char 	waveCald [WAVECALDLEN];
  char 	delayCalSpeed [DLYCALSPDLEN];
  int	  recalRad;			// recalibrate indicators
	int		recalWave;
  int   status;				// returned dialogue status
public:
		__fastcall XLCalData(TComponent *Owner);
		__fastcall ~XLCalData();
		BOOL  Invalid();
		char* getInRad()		{ return (inRad); }
		char* getOutRad()		{ return (outRad); }
		char* getInCount()	{ return (inCount); }
		char* getOutCount()	{ return (outCount); }
		char* getInLimit()	{ return (inLimit); }
		char* getOutLimit()	{ return (outLimit); }
		char* getRadCald()	{ return (radCald); }
		int   getRecalRad()	{ return (recalRad); }
		int   getRecalWave()	{ return (recalWave); }
		char* getLowWave(BOOL protect = FALSE)	{
						char wave [LOWWAVELEN];
            char * pwave = &wave[0];
						(protect) ?
							strcpy (wave, protectedLowWave):
							strcpy (wave, lowWave);
						return pwave; }
		char* getHiWave(BOOL protect = FALSE)		{
						char wave [HIWAVELEN];
            char * pwave = &wave[0];
						(protect) ?
							strcpy (wave, protectedHiWave):
							strcpy (wave, hiWave);
						return pwave; }
		char* getLowCount()	{ return (lowCount); }
		char* getHiCount()	{ return (hiCount); }
		char* getLowLimit()	{ return (lowLimit); }
		char* getHiLimit()	{ return (hiLimit); }
		char* getWaveCald()	{ return (waveCald); }
		char* getDelayCalSpeed()	{ return (delayCalSpeed); }
		char* getCollarOffset()	{ return (collarOffset); }
		int		getStatus()		{ return (status); }
		void 	getAll (XLCalData* source, BOOL protect = FALSE)
      {///Get all of the calibration data
        putInRad (source->getInRad());
        putOutRad (source->getOutRad());
        putLowWave (source->getLowWave(), protect);
        putHiWave (source->getHiWave(), protect);
        putRecalRad (source->getRecalRad());
        putRadCald (source->getRadCald());
        putRecalWave (source->getRecalWave());
        putWaveCald (source->getWaveCald());
      }
		void  putInRad (char* data)			{
						(strcmp (data, "") == 0) ?
							strcpy (inRad, DEFAULT_INRAD):
							strcpy (inRad, data); }
		void  putOutRad (char* data)		{
						(strcmp (data, "") == 0) ?
							strcpy (outRad, DEFAULT_OUTRAD):
							strcpy (outRad, data); }
		void  putInCount (char* data)		{
						(strcmp (data, "") == 0) ?
							strcpy (inCount, DEFAULT_INCOUNT):
							strcpy (inCount, data); }
		void  putOutCount (char* data)	{
						(strcmp (data, "") == 0) ?
							strcpy (outCount, DEFAULT_OUTCOUNT):
							strcpy (outCount, data); }
		void  putInLimit (char* data)		{
						(strcmp (data, "") == 0) ?
							strcpy (inLimit, DEFAULT_INLIMIT):
							strcpy (inLimit, data); }
		void  putOutLimit (char* data)	{
						(strcmp (data, "") == 0) ?
							strcpy (outLimit, DEFAULT_OUTLIMIT):
							strcpy (outLimit, data); }
		void  putRadCald (char* data)		{
						(strcmp (data, "") == 0) ?
							strcpy (radCald, DEFAULT_RADCALD):
							strcpy (radCald, data); }
		void  putRecalRad (int data)		{ recalRad = data; }
		void  putRecalWave (int data)		{ recalWave = data; }
		void  putLowWave (char* data, BOOL protect = FALSE)		{
						(strcmp (data, "") == 0) ?
							strcpy (lowWave, DEFAULT_LOWWAVE):
							strcpy (lowWave, data);
						if (protect)
							strcpy (protectedLowWave, lowWave); }
		void  putHiWave (char* data, BOOL protect = FALSE)		{
						(strcmp (data, "") == 0) ?
							strcpy (hiWave, DEFAULT_HIWAVE):
							strcpy (hiWave, data);
						if (protect)
							strcpy (protectedHiWave, hiWave); }
		void  putLowCount (char* data)	{
						(strcmp (data, "") == 0) ?
							strcpy (lowCount, DEFAULT_LOWCOUNT):
							strcpy (lowCount, data); }
		void  putHiCount (char* data)		{
						(strcmp (data, "") == 0) ?
							strcpy (hiCount, DEFAULT_HICOUNT):
							strcpy (hiCount, data); }
		void  putLowLimit (char* data)	{
						(strcmp (data, "") == 0) ?
							strcpy (lowLimit, DEFAULT_LOWLIMIT):
							strcpy (lowLimit, data); }
		void  putHiLimit (char* data)		{
						(strcmp (data, "") == 0) ?
							strcpy (hiLimit, DEFAULT_HILIMIT):
							strcpy (hiLimit, data); }
		void  putWaveCald (char* data)		{
						(strcmp (data, "") == 0) ?
							strcpy (waveCald, DEFAULT_WAVECALD):
							strcpy (waveCald, data); }
		void  putDelayCalSpeed (char* data) {
						(strcmp (data, "") == 0) ?
							strcpy (delayCalSpeed, DEFAULT_DLYCALSPD):
							strcpy (delayCalSpeed, data); }
		void  putCollarOffset (char* data) {
						(strcmp (data, "") == 0) ?
							strcpy (collarOffset, DEFAULT_COLLAROFFSET):
							strcpy (collarOffset, data); }
		void 	putStatus (int dlgStatus)  { status = dlgStatus; }
};
//---------------------------------------------------------------------------
#endif
