#include <vcl.h>
#pragma hdrstop

#ifndef XLCommInH
#define XLCommInH
#include "XLCommOut.h"
#include "XLDataFile.h"
#include "XLData.h"
//---------------------------------------------------------------------------

class XLCommIn:public XLCommOut
{///All messages to the XLI are handled here
private:
  void  StoreElapsedTime(char* inmsg);
  void  StoreSettings(char* inmsg);
  void  StoreError(char* inmsg);
  void  DisplayError(int error, int index);	                                  ///Handles XL diag numbers
  void  StoreStatus(char* inmsg);
  bool  VerifyCheckSum(char* inmsg);
  void  StoreData(char * inmsg);                                              ///Handle case when data is being sent over
	bool __fastcall SaveFsScanData(char * inmsg);                               ///Handle field service scan data
  bool __fastcall StoreFS(char * inmsg);                                      ///Store the Field service data
  bool __fastcall StoreCalibration (char * inmsg);                            ///Store the calibration factors
	bool __fastcall SaveScanData(char * inmsg);                                 ///Save a Beckman scan data file- return true if successful
  void  DecodeDABError(char* inmsg);                                          ///Get the string corresponding to the DAB error and add it to the ErrLog
 ///Functions to crack an XLA message into its component pieces
  float getRadius(char *inmsg);                                               ///returns for wavelength scans
  float getRadius(char * inmsg,int index);                                    ///Returns radius for other types of scans
  long  getTime(char * inmsg);                                                ///Return the run time from the XLA message
  float getW2tMantissa(char * inmsg);                                         ///Returns the w2t mantissa as a floating point number
  int   getW2tExponent(char* inmsg);                                          ///Returns the exponent of w2t
  int   getSpeed(char * inmsg);                                               ///Returns the rotor speed in thousands
  float getTemp(char * inmsg);                                                ///Returns the temperature as xx.x
  float getWave(char * inmsg);                                                ///Get wavelength for radial scans
  float getWave(char * inmsg,int index);                                      ///For wavelength scan... offset calculated here
  float getStdDev (char * inmsg,int index);                                   ///Get the standard deviation on the absorbance
  float getRefSig (char * inmsg,int index);                                   ///Get the signal for the reference sector
  float getSampleSig (char * inmsg,int index);                                ///Returns the sample intensity
  float getIncSig (char * inmsg,int index);                                   ///Returns the incident light signal
  float getOd(char * inmsg,int index);                                       ///Returns the value of the optical density from the msg stream
  int   getPmtV (char * inmsg,int index);                                     ///What is the voltage on the absorbance PMT
  int   getPgaGain (char * inmsg,UINT index);                                 ///Returns the setting of the Beckman PGA
  float getSampDelay (char * inmsg,int index);                                ///Get the delay time for flashing the sample sector
  float getRefDelay (char * inmsg,int index);                                 ///Return the angle for flashing the reference sector
  float getDelaySig (char * inmsg,int index);                                ///Get the ???
  float getDelayTime (char * inmsg,int index, int etxPos);                    ///Returns the time for the delay to some position...
  bool  InitXLDataFile(char * inmsg,XLDataFile * xldf);                       ///Create a Beckman style header line 2 from the inmsg and fill in the XLDataFile info
  int MsgSize;                                                                ///Number of bytes in the inmsg
  bool __fastcall CheckEndMsg(char * inmsg, int index);                       ///Returns true if there is no more data in inmsg
	public:
	__fastcall XLCommIn(TComponent *Owner);
	__fastcall ~XLCommIn();
 /// Used to determine how fast the machine speed is changing
	int LastRpm;
 ///Handle incoming messages
	XLData *XLActual;                                                           ///Actual values of temperature, pressure, rpm, w^2t from the XL
	XLHWData *OpticsActual;                                                     ///Holds the desired hw settings for the XL optical systems
  __fastcall void MessageDecoder(char* RxBuffer,int msgsize);                 ///RxBuffer contains the message, MsgSize is the number of bytes
  void XLRequestMessage(char* inmsg);
};
//---------------------------------------------------------------------------
#endif
