//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "XLCommOut.h"
#include <stdio.h>

#include "XLScanControl.h"
//---------------------------------------------------------------------------
__fastcall XLCommOut::XLCommOut(TComponent *Owner):baXLComm(Owner)
{
  XLDesired         = new XLData(this);                           ///These are the desired rpm, temperature, run time, acceleration, etc.
	XLDesired->XMLDoc = "MachineDesired";														/// The name of the XMLDocument containing the settings
	OpticsDesired     = new XLHWData(this);                         ///These are the desired settings for the XLA settings
  CalSettings       = new XLCalData(this);                        ///Calibration settings
	VacuumSystem    = new TVacuumSystem(this);      								///The machine object has a vacuum system
	TemperatureSystem= new TTemperatureSystem(this);								///The machine object has a temperature system
}
//---------------------------------------------------------------------------
__fastcall XLCommOut::~XLCommOut()
{
  if(XLDesired)     delete XLDesired;
  if(OpticsDesired) delete OpticsDesired;
  if(DlyScanSettings) delete DlyScanSettings;                     ///Settings needed for delay scan calibration
  if(WCalScanSettings) delete WCalScanSettings;                   ///Settings needed for wavelength scan calibration
  if(TimeScanSettings) delete TimeScanSettings;                   ///Settings needed for time scan calibration
	if(CalSettings) delete CalSettings;                             ///Calibration settings
	if(VacuumSystem)      delete VacuumSystem;											/// Vacuum system
	if(TemperatureSystem) delete TemperatureSystem;                 /// Temperature system

}
//---------------------------------------------------------------------
void __fastcall XLCommOut::CreateCalScan()
{///Have to create the CalScanData objects after Machine is valid
  DlyScanSettings   = new XLCalScanData(this,DELAY_SCAN);         ///Settings needed for delay scan calibration
  WCalScanSettings  = new XLCalScanData(this,WAVE_SCAN);          ///Settings needed for wavelength scan calibration
  TimeScanSettings  = new XLCalScanData(this,TIME_SCAN);          ///Settings needed for time scan calibration
}
//---------------------------------------------------------------------
bool XLCommOut::SendXLMessage(char* msg)
{///Send a message to the XL
  char outmsg[256];
  memset(outmsg, 0, sizeof(outmsg));
  bool okay = true;
  int i;
  outmsg[0] = 2;			// set STX & ETX
  outmsg[5] = 3;
  switch (msg[0])
    {///Switch on the first character
      case 'R':                                                               ///Request data from the XL
        okay = CreateRequest(msg+1,outmsg);
        break;
      case 'C':                                                               ///Command sent to the XL
        okay = CreateCommand(msg,outmsg);
        break;
      case 'S':
        switch (msg[1])                                                       ///What happens depends on the second character
          {///Settings message
            case 'S':
              okay = CreateXlSettings(msg, outmsg);
              break;
              case 'R' :
              case 'W' : okay = CreateScanSettings (msg, outmsg);
                break;
              case 'T' : okay = CreateDelayScanSettings (msg, outmsg);
                break;
              case 'I' : okay = CreateTimeScanSettings (msg, outmsg);
                break;
              case 'Q' : okay = CreateWCalPeakSettings (msg, outmsg);
                break;
            default:
              okay = FALSE;		// nonsensical
              break;
          }
        break;
      case 'Z':
        CreateCalParms(outmsg);                                               ///Create a set of the calibration parameters
        break;
      case 'H' : okay = CreateHwSettings (msg, outmsg);
        break;
      default:
        okay = FALSE;
        break;
    }
  if (okay)
    {///The message was created correctly, so send it
      InsertCksum(outmsg);
      Write(outmsg,strlen(outmsg));
      if(MsgStatus) MsgStatus->AddFrom(outmsg);
    }
	return true;
}
//---------------------------------------------------------------------
bool XLCommOut::CreateHwSettings(char* cmd, char outmsg[])
{///Create hardware settings for the XL optical systems
	char temp [10];
	float fVal;
	short i, pgaGain, cellNum = -1;
	long delayComp;
	int status = 1;
	outmsg [1] = cmd [0];					// 'H' character
	outmsg [2] = cmd [1];					// command character

	switch (cmd [1]) {
		case 'O':			// rotor hole command
			outmsg [3] = cmd [2];
			break;
		case 'H':		                ///Laser delay- do not handle Beckman interference system
			break;
		case 'F':		                ///Laser duration- do not handle Beckman interference system
			break;
		case 'J':		                ///Laser firings per cell- do not handle Beckman interference system
			break;
		case 'K':		                ///Laser off/on command- do not handle Beckman interference system
			break;
		case 'M':		                ///Field service diagnostic scan enable/disable
			strcpy (outmsg + 3, cmd + 2);
			break;
		case 'P':		                ///PMT voltage
			switch (cmd [2])
       {///Different modes of handling PMT voltage setting
          case 'A':			        /// force/release PMT use values
          case 'Z':             /// enable/disable diagnostic scan
            strcpy (temp, cmd + 2);
            break;
          default:			        /// send requested value
            sprintf (temp, "%04d", atoi (OpticsDesired->getPmtVoltage()));
            break;
  			}
			strcpy (outmsg + 3, temp);
			break;
		case 'S':		                /// PGA gain; 1=1, 2=2, 3=4, 4=8, 5=16, 6=32, 7=64, 8=128, 9=256
			switch (cmd [2])
       {///Different modes for setting gain
        case 'A':			// force/release PMT value use
        case 'Z':
          strcpy (temp, cmd + 2);
          break;
        default:			// send requested value
          pgaGain = atoi (OpticsDesired->getPgaGain());
          i = 0;
          do
            {
              pgaGain = pgaGain >> 1;
              i++;
            } while (pgaGain != 0);
          pgaGain = 1 << (i - 1);
          sprintf (temp, "%03d", pgaGain);
          break;
  			}
			strcpy (outmsg + 3, temp);
			break;
		case 'I':		// incident gain
			switch (cmd [2]) {
				case 'A':			// force/release PMT value use
				case 'Z':
					strcpy (temp, cmd + 2);
					strcpy (outmsg + 3, temp);
					break;
				default:			// send requested value
					strcpy (temp, OpticsDesired->getIncGain());
					outmsg [3] = temp [0];
					break;
			}
			break;
//		case 'C':		// integration time
//			sprintf (temp, "%02d", atoi (OpticsDesired->getIntTime()));
//			strcpy (outmsg + 3, temp);
//			break;
		case 'D':		// radial position
			fVal = atof (OpticsDesired->getRadPos()) * 1000.0;
			sprintf (temp, "%04d", (int) fVal);
			strcpy (outmsg + 3, temp);
			break;
		case 'W':		// wavelength position
			sprintf (temp, "%03d", atoi (OpticsDesired->getWavePos()));
			strcpy (outmsg + 3, temp);
			break;
		case 'A':		// strobe rate
			sprintf (temp, "%05ld", OpticsDesired->getStrobeRate());
			strcpy (outmsg + 3, temp);
			break;
		case 'G':		// strobe cell
			strcpy (temp, OpticsDesired->getStrobeCell());
			(temp [0] == '0') ?
				outmsg [3] = 'S':
				outmsg [3] = temp [0];
			break;
		case 'R':		// radial rate
			fVal = atof (OpticsDesired->getRadSpeed()) * 100.0;
			sprintf (temp, "%03d", abs ((int) fVal));
			strcpy (outmsg + 3, temp);
			(fVal < 0.0) ?
				outmsg [6] = '0':
				outmsg [6] = '1';
			break;
		case 'L':		// wavelength rate
			fVal = atof (OpticsDesired->getWaveSpeed()) * 10.0;
			sprintf (temp, "%03d", abs ((int) fVal));
			strcpy (outmsg + 3, temp);
			(fVal < 0.0) ?
				outmsg [6] = '0':
				outmsg [6] = '1';
			break;
		case 'N':		// diagnostic tolerance
			for (i = 0; i < 2; i++)
				outmsg [i + 3] = cmd [i + 2];
			break;
		case 'B': 	// radial calibration positions
			fVal = atof (CalSettings->getInRad()) * 1000.0;
			sprintf (temp, "%04d", (int) fVal);
			strcpy (outmsg + 3, temp);
			fVal = atof (CalSettings->getOutRad()) * 1000.0;
			sprintf (temp, "%04d", (int) fVal);
			strcpy (outmsg + 7, temp);
			break;
		case 'E': 	// wavelength calibration positions
			sprintf (temp, "%03d", atoi (CalSettings->getLowWave (PROTECTED)));
			strcpy (outmsg + 3, temp);
			sprintf (temp, "%03d", atoi (CalSettings->getHiWave (PROTECTED)));
			strcpy (outmsg + 6, temp);
			break;
		default:
			MachineDB->AddError(1006,"XLCommOut::CreateHwSettings- Undefined HW command");
			break;
	}
	return status;
}
//---------------------------------------------------------------------
int XLCommOut::CreateRequest(char* reqType, char* outmsg)
{///Create a request for data from the XL
	switch (reqType[0]){
  	case 'S' : 	CreateReqSettings(outmsg);		        ///Request Desired XLsettings
								break;
		case 'X' : 	CreateReqActuals(outmsg);				      ///Request actual status
								break;
		case 'I':   RequestElapsedTime(outmsg);		        ///Reuest elapsed time
			          break;
	}
	return 1;
}
//---------------------------------------------------------------------
int XLCommOut::CreateReqSettings(char* outmsg)
{///Will request the current value of the Desired setttings
  outmsg[0] = STX;
	outmsg[1] = 'R';
	outmsg[2] = 'S';
	return 3;
}
//---------------------------------------------------------------------
int XLCommOut::CreateReqActuals(char* outmsg)
{///Will request the actual values for  rpm, temp, etc.
  outmsg[0] = STX;
	outmsg[1] = 'R';
	outmsg[2] = 'X';
	return 3;
}
//---------------------------------------------------------------------
int XLCommOut::RequestElapsedTime(char* outmsg)
{///Will request the elapsed time
  outmsg[0] = STX;
	outmsg[1] = 'R';
	outmsg[2] = 'I';
	return 3;
}
//---------------------------------------------------------------------
int XLCommOut::CreateCommand(char* cmd, char* outmsg)
{///Create a command to go to the XL... characters 0,1 and 2 tell that this is a command
	outmsg[0] = STX;
	outmsg[1] = cmd[0];
  outmsg[2] = cmd[1];
	switch (cmd[1])
    {///Create the correct command depending on the second character
      case 'L' : CreateCmdStart(outmsg);	                                    /// start XL
        break;
      case 'X' : CreateCmdStop(outmsg);		                                    /// stop XL
        break;
      case 'C' :													                                    /// recalibrate command
        outmsg[3] = cmd[2];
        break;
      default:
        break;
    }
  return 1;
}
//---------------------------------------------------------------------
int XLCommOut::CreateCmdStart(char* outmsg)
{///Append the correct character to get the XL to Start
	outmsg[3] = 'A';
	return 4;
}
//---------------------------------------------------------------------
int XLCommOut::CreateCmdStop(char* outmsg)
{///The stop command has no modifications made to it
  return 3;
}
//---------------------------------------------------------------------
int XLCommOut::CreateXlSettings(char* msgtype, char* outmsg)
{
	char temp[80];
	short mins, hrs;
	outmsg[0] = STX;
	outmsg[1] = msgtype[0];
  outmsg[2] = msgtype[1];
 	// speed
  short i;
	sprintf(temp, "%3.3d", atol(XLDesired->getSpeed()) / 100);
	for (i = 0; i < 3; i++) outmsg[i+3] = temp[i];

 	// time
	(XLDesired->getTimeHHHMM() == 0) ?
  	sprintf(temp, "%s", "HHHHH\0"):
  	sprintf(temp, "%05ld", XLDesired->getTimeHHHMM());
	for (i = 0; i < 5; i++) outmsg[i+6] = temp[i];

 	// temperature
	int temperature = (int) (atof(XLDesired->getTemp()) * 10.0);
	sprintf(temp, "%3.3d", temperature);
	for (i = 0; i < 3; i++) outmsg[i+11] = temp[i];

 	// w2t
	if (atoi(XLDesired->getW2t()) == 0) strcpy(temp, "XXXXX");
	else strcpy (temp, XLDesired->getW2t());
	for (i = 0; i < 5; i++) outmsg[i+14] = temp[i];

 	// XL accel
	strcpy(temp, XLDesired->getAccel());
	outmsg[19] = temp[0];

 	// XL decel
	strcpy(temp, XLDesired->getDecel());
	outmsg[20] = temp[0];

 	// Analytical accel/decel
	sprintf(temp, "%3.3d", atoi(XLDesired->getAnDecel()));
	for (i = 0; i < 3; i++) outmsg[i+21] = temp[i];

	return 1;
}

//---------------------------------------------------------------------
void XLCommOut::CreateCalParms(char* outmsg)
{
	char temp[4];
  TStringList *TempList = new TStringList; // declare the list
	outmsg[0] = STX;
	outmsg[1] = 'S';
	outmsg[2] = 'C';
	outmsg[3] = NULL;
  try
    {//use the string list
      AnsiString AVIPath=ExtractFilePath(Application->ExeName);
      AVIPath=AVIPath+"CAL_PARM.CAL";
      TempList->LoadFromFile(AVIPath);
      strcat(outmsg,TempList->Strings[0].c_str());
    }
  __finally
    {
      delete TempList; // destroy the list object
    }
}
//*******************************************************************
bool XLCommOut::CreateScanSettings(char* msgtype, char* outmsg)
{
	char temp [20];
	int index = 2;
  bool okay = false;
	outmsg[0] = STX;
	outmsg[1] = 'S';
	for (short cell = 0; cell < NUM_CELLS; cell++) {
		strcpy (temp, XLScanController->XLScanner->getTypeScan(cell));
		switch (temp[0]) {
			// **************** R A D I A L   S C A N ********************
			case 'R':
				okay = true;
				outmsg[index++] = temp [0];
				sprintf (temp, "%04.0f",
									atof (XLScanController->XLScanner->getRMin(cell))* 1000.0);
				outmsg[index++] = temp[0];		// 1; for checking against DAB input code
				outmsg[index++] = temp[1];
				outmsg[index++] = temp[2];
				outmsg[index++] = temp[3];
				sprintf (temp, "%04.0f",
									atof (XLScanController->XLScanner->getRMax(cell))* 1000.0);
				outmsg[index++] = temp[0];		// 5
				outmsg[index++] = temp[1];
				outmsg[index++] = temp[2];
				outmsg[index++] = temp[3];
				sprintf (temp, "%03d",
									atoi (XLScanController->XLScanner->getWMin(cell)));
				outmsg[index++] = temp[0];		// 9
				outmsg[index++] = temp[1];
				outmsg[index++] = temp[2];
				sprintf (temp, "%03d",
									atoi (XLScanController->XLScanner->getW2(cell)));
				outmsg[index++] = temp[0];		// 12
				outmsg[index++] = temp[1];
				outmsg[index++] = temp[2];
				sprintf (temp, "%03d",
									atoi (XLScanController->XLScanner->getW3(cell)));
				outmsg[index++] = temp[0];		// 15
				outmsg[index++] = temp[1];
				outmsg[index++] = temp[2];
				sprintf (temp, "%03.0f",
									atof (XLScanController->XLScanner->getRStep(cell))* 1000.0);
				outmsg[index++] = temp[0];		// 18
				outmsg[index++] = temp[1];
				outmsg[index++] = temp[2];
				strcpy (temp, XLScanController->XLScanner->getScanMode(cell));
				outmsg[index++] = temp[0];		// 21
				sprintf (temp, "%03d",
									atoi (XLScanController->XLScanner->getReplicates(cell)));
				outmsg[index++] = temp[0];		// 22
				outmsg[index++] = temp[1];
				outmsg[index++] = temp[2];
				strcpy (temp, XLScanController->XLScanner->getOutputMode(cell));
				outmsg[index++] = temp[0];		// 25
				strcpy (temp, XLScanController->XLScanner->getOutputType(cell));
				outmsg[index++] = temp[0];		// 26
				strcpy (temp, XLScanController->XLScanner->Channels(cell));
				outmsg[index++] = temp[0];		// 26
				break;
			// **************** W A V E L E N G T H   S C A N ********************
			case 'W':
				okay = true;
				outmsg[index++] = temp [0];
				sprintf (temp, "%04.0f",
									atof (XLScanController->XLScanner->getRMin(cell))* 1000.0);
				outmsg[index++] = temp[0];		// 1; for checking against DAB input code
				outmsg[index++] = temp[1];
				outmsg[index++] = temp[2];
				outmsg[index++] = temp[3];
				sprintf (temp, "%04.0f",
									atof (XLScanController->XLScanner->getRMax(cell))* 1000.0);
				outmsg[index++] = temp[0];		// 5
				outmsg[index++] = temp[1];
				outmsg[index++] = temp[2];
				outmsg[index++] = temp[3];
				sprintf (temp, "%03.0f",
									atof (XLScanController->XLScanner->getRStep(cell))* 100.0);
				outmsg[index++] = temp[0];		// 9
				outmsg[index++] = temp[1];
				outmsg[index++] = temp[2];
				sprintf (temp, "%03d",
									atoi (XLScanController->XLScanner->getWMin(cell)));
				outmsg[index++] = temp[0];		// 12
				outmsg[index++] = temp[1];
				outmsg[index++] = temp[2];
				sprintf (temp, "%03d",
									atoi (XLScanController->XLScanner->getWMax(cell)));
				outmsg[index++] = temp[0];		// 15
				outmsg[index++] = temp[1];
				outmsg[index++] = temp[2];
				sprintf (temp, "%03d",
									atoi (XLScanController->XLScanner->getWStep(cell)));
				outmsg[index++] = temp[0];		// 18
				outmsg[index++] = temp[1];
				outmsg[index++] = temp[2];
				strcpy (temp, XLScanController->XLScanner->getScanMode(cell));
				outmsg[index++] = temp[0];		// 21
				sprintf (temp, "%03d",
									atoi (XLScanController->XLScanner->getReplicates(cell)));
				outmsg[index++] = temp[0];		// 22
				outmsg[index++] = temp[1];
				outmsg[index++] = temp[2];
				strcpy (temp, XLScanController->XLScanner->getOutputMode(cell));
				outmsg[index++] = temp[0];		// 25
				strcpy (temp, XLScanController->XLScanner->getOutputType(cell));
				outmsg[index++] = temp[0];		// 26
				strcpy (temp, XLScanController->XLScanner->Channels(cell));
				outmsg[index++] = temp[0];		// 26
				break;
			// ************************* N O  S C A N *******************************
			default:			// this be a no-scanner
				outmsg[index++] = 'X';
				break;
		}	// end switch
	}	// end for
	outmsg[index++] = XLScanController->XLScanner->NoDelayCal();
	sprintf (temp, "%03d",
						atoi (XLScanController->XLScanner->FilterLevel()));
	outmsg[index++] = temp[0];		// 15
	outmsg[index++] = temp[1];
	outmsg[index++] = temp[2];
	return okay;
}
//*******************************************************************
bool XLCommOut::CreateDelayScanSettings(char* msgtype, char* outmsg)
{
	char temp [5];

	outmsg[0] = STX;
	outmsg[1] = 'S';
	outmsg[2] = 'T';
	strcpy (temp, DlyScanSettings->getItem (0));
	outmsg [3] = temp [0];		// cell number
	sprintf (temp, "%04.0f", atof (DlyScanSettings->getItem (2)) * 1000);
	outmsg [4] = temp [0]; 		// radius * 1000
	outmsg [5] = temp [1];
	outmsg [6] = temp [2];
	outmsg [7] = temp [3];
	sprintf (temp, "%03d", atoi (DlyScanSettings->getItem (3)));
	outmsg [8] = temp [0]; 		// wavelength
	outmsg [9] = temp [1];
	outmsg [10] = temp [2];
	outmsg [11] = 'A';				// output at completion
	outmsg[12] = NULL;
  return true;
}
//*******************************************************************
bool XLCommOut::CreateTimeScanSettings(char* msgtype, char* outmsg)
{
	char temp [5];
	outmsg[0] = STX;
	outmsg[1] = 'S';
	outmsg[2] = 'I';
	strcpy (temp, TimeScanSettings->getItem (0));
	outmsg [3] = temp [0];		// cell number
	sprintf (temp, "%04.0f", atof (TimeScanSettings->getItem (2)) * 1000);
	outmsg [4] = temp [0]; 		// radius * 1000
	outmsg [5] = temp [1];
	outmsg [6] = temp [2];
	outmsg [7] = temp [3];
	sprintf (temp, "%03d", atoi (TimeScanSettings->getItem (3)));
	outmsg [8] = temp [0]; 		// wavelength
	outmsg [9] = temp [1];
	outmsg [10] = temp [2];
	outmsg [11] = 'A';				// output at completion
	outmsg[12] = NULL;
  return true;
}
//*******************************************************************
//								C r e a t e W C a l S c a n
//*******************************************************************
bool XLCommOut::CreateWCalScanSettings(char outmsg[])
{
	char temp [10];
	strcpy (temp, WCalScanSettings->getItem (0));
	outmsg [3] = temp [0];		// cell number
	sprintf (temp, "%04.0f", atof (WCalScanSettings->getItem (3)) * 1000);
	outmsg [4] = temp [0]; 		// radius * 1000
	outmsg [5] = temp [1];
	outmsg [6] = temp [2];
	outmsg [7] = temp [3];
	outmsg [8] = NULL;
  return true;
}
//---------------------------------------------------------------------
bool XLCommOut::CreateWCalPeakSettings (char cmd[], char outmsg[])
{
	char temp [10], peak [4];
	outmsg [1] = cmd [0];
	outmsg [2] = cmd [1];
	strcpy (peak, XLScanController->LoCalPeak());
	strcpy (outmsg + 3, peak);
	strcpy (peak, XLScanController->HiCalPeak());
	strcpy (outmsg + 6, peak);
	outmsg [9] = '\0';
  return true;
}
//---------------------------------------------------------------------
//									Checksum functions
//---------------------------------------------------------------------
void XLCommOut::InsertCksum(char* outmsg){
  int sum = 0;
	int length = strlen(outmsg);

	for (int i = 0; i < length; i++) sum += outmsg[i];
	outmsg[length] = ((sum / 10) % 10) + 48;
  outmsg[length+1] = sum % 10 + 48;
	outmsg[length+2] = ETX;
}
//---------------------------------------------------------------------------

#pragma package(smart_init)
