//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "XLCommSettings.h"
#include "Machine.h"
#include "Utility.h"
#include "EnumeratePorts.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"


TCommSet *CommSet;
//---------------------------------------------------------------------------
__fastcall TCommSet::TCommSet(TComponent* Owner)
				: TForm(Owner)
{/// Info needed to handle the XL COM port
	LoadPortBox();
	AnsiString s=Machine->Port;
	bOK   = true;
	LastXLCmd = "RS";
	UpdateXLData();
	bNeedsUpdate = false;
}
//---------------------------------------------------------------------------
void __fastcall TCommSet::LoadPortBox()
{/// Load the port box with the available com ports
	XLPortBox->Clear();
	EnumeratePorts * ef = new EnumeratePorts(this);
	for (int i=0; i < ef->numports; i++){
		AnsiString t =ef->PortNames->Strings[i];
		if((t.SubString(1,3)).LowerCase() == "com"){
    	XLPortBox->AddItem(t,NULL);
		}
	}
	delete ef;
	AnsiString s = Machine->Port+":";
	int i = XLPortBox->Items->IndexOf(s);
	if(i >= 0){          																												///Can select the current port from the list?
		XLPortBox->ItemIndex = i;
	}
	else{
		XLPortBox->ItemIndex = 0;
  }
}
//---------------------------------------------------------------------------
void __fastcall TCommSet::OKbtnClick(TObject *Sender)
{//Make it show which port is not opening
	AnsiString s;
	if(!Machine->IsOpen)
	{
		if (!bOK) s=s+" and the ";
		s=s+"XL COM port not open";
		bOK=false;
	}
	Close();
}
//---------------------------------------------------------------------------
void __fastcall TCommSet::ClearMemoClick(TObject *Sender)
{
	Memo1->Clear();
}
//---------------------------------------------------------------------------
void TCommSet::UpdateCommSet(AnsiString s)
{
	 Memo1->Lines->Add(s);
}
//---------------------------------------------------------------------------
void __fastcall TCommSet::FormShow(TObject *Sender)
{
	ComStatTxt->Caption = Machine->ConnectionState();
	UpdateXLData();
}
//---------------------------------------------------------------------------
void __fastcall TCommSet::XLPortBoxChange(TObject *Sender)
{
	Machine->CloseCommPort();          																					/// Kill thread and close handle
	AnsiString s=XLPortBox->Text;
	int i = s.Pos(":");
	s = s.SubString(1,i-1);																											/// Extract the COMx string
	Machine->SetCommPort(s);
	s=s+ ": open XL port failure.";
#define OPENCOMM		  					50
	if(!Machine->InitializeCommPort()){
		Machine->MachineDB->AddError(OPENCOMM,s);																	/// Use until the error code situation is better defined
	}
	ComStatTxt->Caption = Machine->ConnectionState();
}
//---------------------------------------------------------------------------
void __fastcall TCommSet::XLSendCommandClick(TObject *Sender)
{
	AnsiString s=XLcb->Text;
	XLastCmdTxt->Caption ="Last Command sent: "+s;
	bool OKtosend=false;
/*
	To do: For the RX and RS commands, set the appropriate SettingsObtained flag
	false. Make sure the CommunicationsEstablished doesn't get called or there will
	be trouble. Use the SettingsObtained to determine what gets put in the memo.
*/
	if(s=="Status")
		{//Send the message to get the XL Status
			s="RX";      //Request settings
			OKtosend=true;
		}
	if(s=="Send calibration parameters")
		{//Send the calibration parameters to the XLA
			s="Z";
			OKtosend=true;
		}
	if(s=="Start")
		{//Send the command to start the fuge
			s="CL";
			OKtosend=true;
		}
	if(s=="Stop")
		{///Send the command to stop the fuge
			s="CX";
			OKtosend=true;
		}
	if(s=="Get current settings")
		{///Send the command to get the current settings
			s="RS";
			OKtosend=true;
		}
	if(OKtosend)
		{///Can send the command
			Machine->SendXLMessage(s.c_str());                                ///Send Message to XL
			XLastCmdTxt->Caption =(XLastCmdTxt->Caption)+" as "+s;
			LastXLCmd = s;
		}
	bNeedsUpdate = true;
	Timer1->Enabled = true;
}
//---------------------------------------------------------------------------
void __fastcall TCommSet::UpdateXLData()
{//Report Status or Settings
	AnsiString s;
	if(LastXLCmd=="RS")
		{// Retrieve settings from fxldata object
			char* temp;
			Memo1->Lines->Add("Desired XL Settings");
			s = "Status: " + AnsiString(Machine->XLDesired->getStatus());
			Memo1->Lines->Add(s);
			s = "Vacuum: " + AnsiString(Machine->XLDesired->getVac());
			Memo1->Lines->Add(s);
			s = "Rotor speed: " + AnsiString(Machine->XLDesired->getSpeed());
			Memo1->Lines->Add(s);
			s = "Time: " + AnsiString(Machine->XLDesired->getXLTime());
			Memo1->Lines->Add(s);
			s = "Temperature: " + AnsiString(Machine->XLDesired->getTemp());
			Memo1->Lines->Add(s);
			s = "w2t: " + AnsiString(Machine->XLDesired->getW2t());
			Memo1->Lines->Add(s);
			s = "Acceleration: " + AnsiString(Machine->XLDesired->getAccel());
			Memo1->Lines->Add(s);
			s = "Deceleration: " + AnsiString(Machine->XLDesired->getDecel());
			Memo1->Lines->Add(s);
			s = "anAcceleration: " + AnsiString(Machine->XLDesired->getAnAccel());
			Memo1->Lines->Add(s);
			s = "anDeceleration: " + AnsiString(Machine->XLDesired->getAnDecel());
			Memo1->Lines->Add(s);
		}
	if(LastXLCmd=="RX")
		{
			Memo1->Lines->Add("Current XL Status");
			s = "Status: " + AnsiString(Machine->XLActual->getStatus());
			Memo1->Lines->Add(s);
			s = "Vacuum: " + AnsiString(Machine->XLActual->getVac());
			Memo1->Lines->Add(s);
			s = "Rotor speed: " + AnsiString(Machine->XLActual->getSpeed());
			Memo1->Lines->Add(s);
			s = "Time: " + AnsiString(Machine->XLActual->getXLTime());
			Memo1->Lines->Add(s);
			s = "Temperature: " + AnsiString(Machine->XLActual->getTemp());
			Memo1->Lines->Add(s);
			s = "w2t: " + AnsiString(Machine->XLActual->getW2t());
			Memo1->Lines->Add(s);
			s = "Acceleration: " + AnsiString(Machine->XLActual->getAccel());
			Memo1->Lines->Add(s);
			s = "Deceleration: " + AnsiString(Machine->XLActual->getDecel());
			Memo1->Lines->Add(s);
			s = "anAcceleration: " + AnsiString(Machine->XLActual->getAnAccel());
			Memo1->Lines->Add(s);
			s = "anDeceleration: " + AnsiString(Machine->XLActual->getAnDecel());
			Memo1->Lines->Add(s);
		}
}
//---------------------------------------------------------------------------
void __fastcall TCommSet::Timer1Timer(TObject *Sender)
{
	if(bNeedsUpdate){
		UpdateXLData();
		bNeedsUpdate = false;
	}
	Timer1->Enabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TCommSet::XLPortBoxKeyDown(TObject *Sender, WORD &Key,
      TShiftState Shift)
{/// Port box is read only
	return;
}
//---------------------------------------------------------------------------

void __fastcall TCommSet::XLPortBoxKeyPress(TObject *Sender, char &Key)
{/// Port box is read only
	return;
}
//---------------------------------------------------------------------------

