//---------------------------------------------------------------------------

#ifndef XLHWDataH
#define XLHWDataH
#include <vcl.h>
#pragma hdrstop
#include <stdio.h>

#define	PMTVLEN 				6
#define	PGAGAINLEN      4
#define	INCGAINLEN      2
#define	DELAYTIMELEN    7
#define	INTTIMELEN      5
#define	RADPOSLEN       6
#define	WAVEPOSLEN      4
#define	STROBERATELEN   6
#define	STROBECELLLEN   2
#define	RADSPEEDLEN     6
#define	WAVESPEEDLEN    6
#define	INCDATALEN      8
#define	RAWDATALEN      8
#define	RESERVED1LEN    2
#define	RESERVED2LEN    2
#define	RESERVED3LEN    2

#define	DEFAULT_PMTV 				"550"
#define	DEFAULT_PGAGAIN     "1"
#define	DEFAULT_INCGAIN     "1"
#define	DEFAULT_DELAYTIME   "\0"
#define	DEFAULT_INTTIME     "4"
#define	DEFAULT_RADPOS      "6.5"
#define	DEFAULT_WAVEPOS     "400"
#define	DEFAULT_STROBERATE  "3000"
#define	DEFAULT_STROBECELL  "0"
#define	DEFAULT_RADSPEED    "1.0"
#define	DEFAULT_WAVESPEED   "5.0"
#define	DEFAULT_INCDATA     "\0"
#define	DEFAULT_RAWDATA     "\0"
#define	DEFAULT_RESERVED1   "\0"
#define	DEFAULT_RESERVED2   "\0"
#define	DEFAULT_RESERVED3   "\0"
#define	DEFAULT_HOLES		    "4"

///Max/min for XLA hardware- all defines are * 1000 so we can work with whole numbers
#define MIN_R			5650.0
#define MAX_R			7300.0
#define MIN_WL		180L
#define MAX_WL		850L
#define MAX_W800	800L
#define MIN_RSPD	10.0
#define MAX_RSPD	2000.0
#define MIN_WSPD	100.0
#define MAX_WSPD	15000.0
#define MIN_RRSTEP	1.0
#define MAX_RRSTEP	50.0
#define MIN_RWSTEP	50.0
#define MAX_RWSTEP	500.0
#define	MIN_TEMP	0.0
#define MAX_TEMP	40000.0
#define MIN_W2T		0.0
#define MAX_W2T   9.99e14
#define MIN_SPEED 1000L
#define MAX_SPEED 60000L
#define MAX_SPEED2 50000L
#define MIN_TIME	1
#define MAX_TIME	9999
#define MIN_PPF		1000.0
#define MAX_PPF		50000.0
#define MIN_ROT		-89000.0
#define MAX_ROT		89000.0
#define MIN_HOLES	4
#define MAX_HOLES 8
#define MIN_FLASH	0
#define MAX_FLASH	999


class XLHWData:public TComponent
{
private:
  char pmtVoltage [PMTVLEN];
  char pgaGain [PGAGAINLEN];
  char incGain [INCGAINLEN];
  char radPos [RADPOSLEN];
  char wavePos [WAVEPOSLEN];
  char strobeRate [STROBERATELEN];
  char strobeCell [STROBECELLLEN];
  char radSpeed [RADSPEEDLEN];
  char waveSpeed [WAVESPEEDLEN];
  char reserved1 [RESERVED1LEN];
  char delayTime [DELAYTIMELEN];
  char rawData [RAWDATALEN];
  char checksum [5];
public:
  __fastcall XLHWData (TComponent* Owner,bool developChecksum = FALSE);
  __fastcall  ~XLHWData();
  void getAll (XLHWData* source);
  void DevelopChecksum(){sprintf (checksum, "%04x", 379);}
  char* CheckSum() { return checksum; }
  char* getPmtVoltage()	{return pmtVoltage;}
  char* getPgaGain() 		{return pgaGain;}
  char* getIncGain() 		{return incGain;}
  char* getDelayTime() 	{return delayTime;}
  char* getRadPos() 		{return radPos;}
  char* getWavePos() 	 	{return wavePos;}
  char* getStrobeRate() {return strobeRate;}
  char* getStrobeCell() {return strobeCell;}
  char* getRadSpeed() 	{return radSpeed;}
  char* getWaveSpeed() 	{return waveSpeed;}
  char* getRawData() 		{return rawData;}
  void putHoles (char* number) { strcpy (reserved1, number); }
  char* getHoles() { return reserved1; }
  void putPmtVoltage (char* data)	{
        (strcmp (data, "") == 0) ?
          strcpy (pmtVoltage, DEFAULT_PMTV):
          strcpy (pmtVoltage, data); }
  void putPgaGain (char* data) 		{
        (strcmp (data, "") == 0) ?
          strcpy (pgaGain, DEFAULT_PGAGAIN):
          strcpy (pgaGain, data); }
  void putIncGain (char* data) 		{
        (strcmp (data, "") == 0) ?
          strcpy (incGain, DEFAULT_INCGAIN):
          strcpy (incGain, data); }
  void putDelayTime (char* data) 	{
        (strcmp (data, "") == 0) ?
          strcpy (delayTime, DEFAULT_DELAYTIME):
          strcpy (delayTime, data); }
  void putRadPos (char* data) 		{
        (strcmp (data, "") == 0) ?
          strcpy (radPos, DEFAULT_RADPOS):
          strcpy (radPos, data); }
  void putWavePos (char* data) 	 	{
        (strcmp (data, "") == 0) ?
          strcpy (wavePos, DEFAULT_WAVEPOS):
          strcpy (wavePos, data); }
  void putStrobeRate (char* data) {
        (strcmp (data, "") == 0) ?
          strcpy (strobeRate, DEFAULT_STROBERATE):
          strcpy (strobeRate, data); }
  void putStrobeCell (char* data)	{
        (strcmp (data, "") == 0) ?
          strcpy (strobeCell, DEFAULT_STROBECELL):
          strcpy (strobeCell, data); }
  void putRadSpeed (char* data)  	{
        (strcmp (data, "") == 0) ?
          strcpy (radSpeed, DEFAULT_RADSPEED):
        strcpy (radSpeed, data); }
  void putWaveSpeed (char* data) 	{
        (strcmp (data, "") == 0) ?
          strcpy (waveSpeed, DEFAULT_WAVESPEED):
          strcpy (waveSpeed, data); }
  void putRawData (char* data) 		{
        (strcmp (data, "") == 0) ?
          strcpy (rawData, DEFAULT_RAWDATA):
          strcpy (rawData, data); }
};
//---------------------------------------------------------------------------
#endif
