//---------------------------------------------------------------------------

#ifndef XLScanControlH
#define XLScanControlH
#include <vcl.h>
#pragma hdrstop
#include <dir.h>
#include "XLScanData.h"
#include <stdio.h>

#define EST_ALL							TRUE
#define	NEW_ROTOR_SCAN			1		// scan setup control indicators
#define SAME_ROTOR_SCAN			0
#define LASERDELAY   				3
#define PROMPT_FSIGNORE			2
#define PROMPT							1
#define NO_PROMPT						0
//Interference scan control states
#define NONE    		        0 			// no int scans
#define ACTIVE    	        2				// int scan is scanning
#define	WAITFORABS          3				// int scan done; waiting for abs scan to finish

class XLScanControl:public TComponent
{///Handles all types of XL scans
private:
  int overlays;
  short rotorHoles;
  int numIntScans;
  short intScanState;
  short currentStep, activeStep;
  long prevSpeed;
  int rotorScanNumber;
  BOOL lastStep, absRequested;
  int lastMethodStep, prevNumScans;
  short hrs, mins, secs;						// for time monitoring
  DWORD tickRef;											// for real-time tick reference
  long fileScanNumber [NUM_CELLS];		// for tracking scans per cell
  short dataCellNum;
  char loCalPeak [4], hiCalPeak [4];
  BOOL dispData, multiCellScan;
  short cellNum;
  char dataFileName [20];						// filename.ext of data
  char lastScanName [MAXPATH];
  char currentPosition [6];
  BOOL scanRequested;
  BOOL xlaIdle;
  BOOL scansFinished;
  int		scanStartType;			// sequential or non-sequential
  UINT	scanIDType;         // actual scan type (string resource ID)
  long	totalDataPts;
  short numScans [NUM_CELLS];				// number of scans for each cell
  long  dataPts [NUM_CELLS];				// number of data points for each cell
  BOOL  delayedStart;       // seq. vel. scan delayed start indicator
  BOOL	timedScan;					// seq. vel. scan time between indicator
  long 	timer;							// seq. vel. timer for timed scans
  int 	rotorScans;
  int 	scansPerRotor;
  BOOL  scanStopped;
  UINT	scanSpeed;					// for equilibrium methods
  int		scanTemp;
  long 	equDelay;						// seq. equ. delay scan time is seconds
  long 	nextEquDelay;				// seq. equ. next delay scan time is seconds
  short equStep;						// current equ-seq step
  BOOL 	scanInProgress;
  BOOL 	needToSendSettings;
  float timeCorrection [NUM_CELLS];
  float tcAvgs [NUM_CELLS];
  long 	startTime;
  BOOL	waitForXlActuals;
  short serviceScanRequested;
  BOOL 	fsScanActive;
  BOOL	dabReset;
  BOOL  stopXlAtEnd;
  BOOL	recalRadius;
  BOOL 	actualsObtained;
  char 	pathName [NUM_CELLS] [MAXPATH];				// path name for each cell
  int 	cellScans [NUM_CELLS];						 		// scans for each cell
  short cellsToScan;
  char 	scanDate [9];
  char 	scanTime [9];
  int 	intScanNum [NUM_CELLS];
public:
  __fastcall XLScanControl(TComponent *Owner);
  __fastcall ~XLScanControl();
  BOOL ScanInProgress();
  XLScanData* XLScanner;
  int CheckOverlays (short type, long scans);
  int Overlays() { return overlays; }
  long FileScanNumber (short cell) { return fileScanNumber [cell]; }
  BOOL AbsRequested() { return absRequested; }
  short IntScanState() { return intScanState; }
  void IntScanState (short state) { intScanState = state; }
  short SetupIntScan (short cell, short type, long* numScans, long* numPts);
  void ControlScan();
  void DataCellNum (short cell) { dataCellNum = cell; }
  char* LoCalPeak() { return loCalPeak; }
  char* HiCalPeak() { return hiCalPeak; }
  void SendWPeaks();
  BOOL GetDispData() { return dispData; }
  int GetCellNum();
  char* getDataFileName() { return dataFileName; }
  BOOL getActualsObtained() {return actualsObtained; }
  void putActualsObtained (BOOL state) { actualsObtained = state; }
  BOOL BuildPathName();
  BOOL getDabReset() { return dabReset; }
  void putDabReset (BOOL state) { dabReset = state; }
  void putScansFinished (BOOL state) { scansFinished = state; }
  BOOL getScansFinished() { return scansFinished; }
  void putXlaIdle (BOOL state) { xlaIdle = state; }
  BOOL getXlaIdle() { return xlaIdle; }		// for timed scan control
  void putScanRequested (BOOL state) { scanRequested = state; }
  BOOL getScanRequested() { return scanRequested; }
  void putLastScanName (char* name) { strcpy (lastScanName, name); }
  char* getLastScanName() { return lastScanName; }
  void putCurrentPosition (char* pos) { strcpy (currentPosition, pos); }
  char* getCurrentPosition() { return currentPosition; }
  UINT 	getScanIDType()			{ return scanIDType; }
  int		getScanStartType()	{ return scanStartType; }
  long  getTotalDataPts()		{ return totalDataPts; }
  BOOL  getDelayedStart()		{ return delayedStart; }
  BOOL  getTimedScan()			{ return timedScan; }
  long  getTimer()					{ return timer; }
  int   getRotorScans()			{ return rotorScans; }
  int   getScansPerRotor()	{ return scansPerRotor; }
  int   getDataPts (short which)	{ return dataPts [which]; }
  BOOL 	getScanStopped()		{ return scanStopped; }
  int   getNumScans (short which)	{ return numScans [which]; }
  int decNumScans (short cell) {
    putNumScans (cell, getNumScans (cell) - 1);
    return getNumScans (cell);
  }
  void  PathName (short index, char* path) { strcpy (pathName [index], path); }
  char* PathName (short index) { return pathName [index]; }
  void 	CellScans (short index, int number) { cellScans [index] = number; }
  int		CellScans (short index) { return cellScans [index]; }
  void  SetDateAndTime();

  void putScanIDType (UINT data) 		{ scanIDType = data; }
  void putScanStartType (int type) 	{ scanStartType = type; }
  void putTotalDataPts (long num) 	{ totalDataPts = num; }
  void putDelayedStart (BOOL state) { delayedStart = state; }
  void putTimedScan (BOOL state) 		{ timedScan = state; }
  void putTimer (long time) 				{ timer = time; }
  void putRotorScans (int scans) 		{ rotorScans = scans; }
  void putScansPerRotor (int scans)	{ scansPerRotor = scans; }
  void putDataPts (short which, int pts) 	{ dataPts [which] = pts; }
  void putScanStopped (BOOL state) 	{ scanStopped = state; }
  void putNumScans (short which, short val) { numScans [which] = val; }
  void putScanSpeed (UINT speed) 		{ scanSpeed = speed; }
  UINT getScanSpeed()								{ return scanSpeed; }
  void putScanTemp (int temp)				{ scanTemp = temp; }
  int getScanTemp()									{ return scanTemp; }
  void putEquDelay (long delay)			{ equDelay = delay; }
  long getEquDelay()								{ return equDelay; }
  void putNextEquDelay (long delay)	{ nextEquDelay = delay; }
  long getNextEquDelay()						{ return nextEquDelay; }
  void putEquStep (short step)			{ equStep = step; }
  short getEquStep()								{ return equStep; }
  void putScanInProgress (BOOL state) { scanInProgress = state; }
  BOOL getScanInProgress()					{ return scanInProgress; }
  void putNeedToSendSettings (BOOL state) { needToSendSettings = state; }
  BOOL getNeedToSendSettings()			{ return needToSendSettings; }
  void putTimeCorrection (short which, float val) {
    timeCorrection [which] = val; }
  void putTimeCorrectionAll (float val) {
    for (short i = 0; i < NUM_CELLS; i++)
      timeCorrection [i] = val; }
  float getTimeCorrection (short which)	{ return timeCorrection [which]; }
  void putWaitForXlActuals (BOOL state) { waitForXlActuals = state; }
  BOOL getWaitForXlActuals()				{ return waitForXlActuals; }
  void putServiceScanRequested (short type) {serviceScanRequested = type; }
  short getServiceScanRequested () {return serviceScanRequested; }
  void putFsScanActive (BOOL state)	{ fsScanActive = state; }
  BOOL getFsScanActive()						{ return fsScanActive; }
  void putStartTime (long sTime) 		{ startTime = sTime; }

  long GetNumScans();
  int CheckDiskSpace (short, short, BOOL = FALSE);
  int ValidateScan();
  int CalcScanTimes (short mode);
  void ReviseEstTimes (BOOL estimateAll = FALSE);
  long GetNumPts (short, short);
  void StartNextScan();
  BOOL SetupNextScan (BOOL);
  void SetupScanSettings();
  void EndOfScan (BOOL actualEnd = TRUE);
  void SendXLSettings();
  void SendXLStart();
  void SendScanSettings();
  void SendScanStart();
  void TimedScanCheck();
  void WaitASecond();
  bool StartScan (int type);
  bool StopScan (BOOL force);
  void ClearControlVars (BOOL enableRaptor = TRUE);
};
//---------------------------------------------------------------------------
#endif
extern XLScanControl* XLScanController;
