//---------------------------------------------------------------------------


#pragma hdrstop
#pragma package(smart_init)

#include "XMLUtilities.h"
#include "AOSSocket.h"
#include "VerInfo.h"
#include <math.h>
#include <dateutils.hpp>
XMLUtilities *XMLUtil;
//---------------------------------------------------------------------------
__fastcall XMLUtilities::XMLUtilities(TComponent *Owner):TComponent(Owner)
{
	TypeStrings = new TStringList();																											///Strings used to test whether a data type is valid
	TypeStrings->Add("string");
	TypeStrings->Add("int");
	TypeStrings->Add("i4");
	TypeStrings->Add("double");
	TypeStrings->Add("boolean");
	TypeStrings->Add("base64");
	TypeStrings->Add("dateTime.iso8601");
	TypeStrings->Add("struct");
	TypeStrings->Add("array");
}
//---------------------------------------------------------------------------
__fastcall XMLUtilities::~XMLUtilities()
{
	delete TypeStrings;
}
//===========================================================================
///GUID Utilities
//---------------------------------------------------------------------------
AnsiString	__fastcall XMLUtilities::NewGUIDString()
{/// Create a unique GUID
	AnsiString s;
	TGUID G;
	CreateGUID(G);
	s = Sysutils::GUIDToString(G);
	return s;
}
//---------------------------------------------------------------------------
AnsiString	__fastcall XMLUtilities::GUIDToString(TGUID uuid)
{/// Returns a String representation of the UUID
	return Sysutils::GUIDToString(uuid);
}
//---------------------------------------------------------------------------
TGUID	__fastcall XMLUtilities::NewGUID()
{/// Returns a GUID
	TGUID G;
	CreateGUID(G);
	return G;
}
//---------------------------------------------------------------------------
TGUID	__fastcall XMLUtilities::StringToGUID(AnsiString s)
{/// Returns a GUID
	return StringToGUID(s);
}
//---------------------------------------------------------------------------
AnsiString	__fastcall XMLUtilities::StringList2String(TStrings * strngs)
{/// Convert a stringlist to a single string
	AnsiString s="";
	for(int i=0;i<strngs->Count;++i){
		s = s+strngs->Strings[i];
	}
	return s;
}
//===========================================================================
///XML Utilities
//---------------------------------------------------------------------------
bool __fastcall XMLUtilities::HasXMLDeclaration(TStrings * strngs)
{///Check to see if the strngs has a valid (for AOS) declaration
	AnsiString s = strngs->Strings[0];
	if(s == XMLDeclaration()) return true;
	return false;
}
//---------------------------------------------------------------------------
bool __fastcall XMLUtilities::OnlyOneXMLDeclaration(TStrings * strngs)
{///Check to see if the strngs only one valid declaration
	bool ret = false;
	AnsiString declar = XMLDeclaration();
	int num;
	for(int i = 0;i<strngs->Count;++i)                                          ///Loop over strngs, increment num every time a declaration shows up
		if(strngs->Strings[i] == declar) ++num;
	if(num == 1) ret = true;
	return ret;
}
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::RemoveAllXMLDeclarations(TStrings * strngs)
{///Remove all XML declarations
	AnsiString declar = XMLDeclaration();
	for(int i = 0;i<strngs->Count;++i)                                          ///Loop over strngs, increment num every time a declaration shows up
		if(strngs->Strings[i] == declar)  ///Found a declaration... delete the string
			strngs->Delete(i);
}
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::FixXMLDeclaration(TStrings * strngs)
{///Make sure there is only one declaration and that it is the first element of strngs
	RemoveAllXMLDeclarations(strngs);
	AddXMLDeclaration(strngs);
}
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::CheckXMLDeclaration(TStrings * strngs)
{///Checks to see if there is a standard XML declaration, and adds one if not
	if(!HasXMLDeclaration(strngs)
		|| !OnlyOneXMLDeclaration(strngs))
		FixXMLDeclaration(strngs);
}
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::AddXMLDeclaration(TStrings * strngs)
{///Adds a standard XML declaration as the first string
	strngs->Insert(0,XMLDeclaration());
}
//===========================================================================
/// XML Wrappers- overloaded function
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLWrap(AnsiString name,double value)
{///Wrap a double as <name>value</name>
	AnsiString s ="<"+name+">"+(AnsiString)value+"</"+name+">";
	return s;
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLWrap(AnsiString name,int value)
{///Wrap an integer as <name>value</name>,
	AnsiString s ="<"+name+">"+(AnsiString)value+"</"+name+">";
	return s;
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLWrap(AnsiString name,AnsiString value)
{///Wrap a string as <name>value</name>,
	AnsiString s ="<"+name+">"+value+"</"+name+">";
	return s;
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLWrap(AnsiString name,bool value)
{///Wrap a bool as <name>value</name>, where value will be 1 = true or 0 = false
	AnsiString tf = 0;
	if(value) tf = "1";
	AnsiString s ="<"+name+">"+tf+"</"+name+">";
	return s;
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLWrap(AnsiString name,AnsiString attr,AnsiString value)
{///Add atributes to name, then wrap it with the string value
	AnsiString nmat = name+attr;
	AnsiString s ="<"+nmat+">"+value+"</"+name+">";
	return s;
}
//===========================================================================
/// XML2Tree and Tree2XML functions
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::XML2Tree(TTreeView * tree,TXMLDocument *XMLDoc)
{///Create a tree view of an XML document
	IXMLNode * jNode;
	tree->Items->Clear();                                                       ///Clear out the tree view
	XMLDoc->Active=true;                                                        ///Make sure the XMLDocument is active... parses the XML
	if(XMLDoc->ChildNodes->First() == NULL) return;                             ///No nodes!
	jNode = XMLDoc->ChildNodes->Nodes[0];
	while(jNode != NULL){
		ProcessNode(jNode,tree, NULL);
		jNode = jNode->NextSibling();
	}
}
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::Tree2XML(TTreeView * treeview,TXMLDocument *XMLDoc)
{///Create a tree view of an XML document
	TTreeNode* tn;
	IXMLNode *iNode;
	XMLDoc->Active = true;
	if(treeview->Items->GetFirstNode() == NULL) return;
	tn = treeview->Items->GetFirstNode();
	iNode = XMLDoc->AddChild(tn->Text);
	while (tn != NULL){																													/// Loop over tree nodes
		ProcessTreeItem(iNode,treeview,tn);
		tn = tn->GetNext();
	}
}
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::TreeExpandAll(TTreeView * node)
{///Expand all nodes in the tree
	TTreeNodes * tns = node->Items;
	if(tns->Count == 0) return;
	for(int i= 0;i<tns->Count;++i)
		tns->Item[i]->Expand(true);
}
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::NodeExpandAll(TTreeNode * node)
{///Expand all child nodes of the node that is passed in
	if(node == NULL || !node->Count == 0) return;
	node->Expand(true);
	for(int i=0;i<node->Count;i++)
		node->Item[i]->Expand(true);
}
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::NodeCollapseAll(TTreeNode * node)
{///Collapse all child nodes of the node that is passed in
	if(node == NULL || !node->Count == 0) return;
	node->Expand(false);
	for(int i=0;i<node->Count;i++)
		node->Item[i]->Collapse(false);
}
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::TreeCollapseAll(TTreeView * node)
{///Collapse all child nodes of the node that is passed in
	TTreeNodes * tns = node->Items;
	if(tns->Count == 0) return;
	for(int i= 0;i<tns->Count;++i)
		tns->Item[i]->Collapse(false);
}
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::ProcessNode(IXMLNode * Node,TTreeView * tree,TTreeNode * tn)
{///Copies info from the XML node to the Tree Node
	if(Node == NULL) return;                                                    ///Can't process an empty node
	IXMLNode * cNode;
	tn = tree->Items->AddChild(tn, Node->NodeName);
	if(Node->AttributeNodes->Count != 0){																				///This is an element node with attributes
		for(int i = 0;i< Node->AttributeNodes->Count;++i){											///Loop over attributes and add their name and value
				IXMLNode * aNode = Node->AttributeNodes->Nodes[i];
				tn->Text = tn->Text+" "+aNode->NodeName+"="+"\""+aNode->Text+"\"";
		}
	}
	if(Node->IsTextElement){                                                     ///This node has only a value, no child nodes
		tn->Text = tn->Text+" "+Node->Text;
		return;
	}
	if(Node->ChildNodes->Count == 0) return;
	cNode = Node->ChildNodes->First();
	while(cNode != NULL){
		ProcessNode(cNode, tree, tn);
		cNode= cNode->NextSibling();
	}
}
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::ProcessTreeItem(IXMLNode * Node,TTreeView * tree,TTreeNode * tn)
{///Copies info from the Tree node to the XML Node
	if(Node == NULL || tn == NULL) return;                                                    ///Can't process an empty node
	IXMLNode * cNode;
	cNode = Node->AddChild(tn->Text);
	if(!tn->HasChildren) return;
	tn = tn->getFirstChild();
	while(tn != NULL){
		ProcessTreeItem(cNode, tree, tn);
		tn= tn->GetNext();
	}
}
//===========================================================================
/// XML Getter functions
//---------------------------------------------------------------------------
int  __fastcall XMLUtilities::GetXMLNodeInteger(IXMLNode * xmlnode,AnsiString name)
{///Returns the integer value for the node named name
	int ret = 0;
	_di_IXMLNodeList Nodes = xmlnode->ChildNodes;
	_di_IXMLNode node = Nodes->FindNode(name);
	if(node == NULL){///No node found
		AnsiString s = "Utility::GetXMLNodeInteger Node: "+name+" not found";
		Application->MessageBox(s.c_str(),"XML error",MB_OK);
		return ret;
	}
	try{																																				///Try to convert to value
		AnsiString s = node->Text;
		ret = s.ToInt();
	}
	catch(...){
		AnsiString s = "Utility::GetXMLNodeInteger Node: "+name+" cannot convert to integer";
		Application->MessageBox(s.c_str(),"XML error",MB_OK);
	}
	return ret;
}
//---------------------------------------------------------------------------
double  __fastcall XMLUtilities::GetXMLNodeDouble(IXMLNode * xmlnode,AnsiString name)
{///Returns the double value for the node named name
	double ret = 0;
	_di_IXMLNodeList Nodes = xmlnode->ChildNodes;
	_di_IXMLNode node = Nodes->FindNode(name);
	if(node == NULL){///No node found
		AnsiString s = "Utility::GetXMLNodeDouble Node: "+name+" not found";
		Application->MessageBox(s.c_str(),"XML error",MB_OK);
		return ret;
	}
	try{																																				///Try to convert to value
		AnsiString s = node->Text;
		ret = s.ToDouble();
	}
	catch(...){
		AnsiString s = "Utility::GetXMLNodeDouble Node: "+name+" cannot convert to double";
		Application->MessageBox(s.c_str(),"XML error",MB_OK);
	}
	return ret;
}
//---------------------------------------------------------------------------
AnsiString  __fastcall XMLUtilities::GetXMLNodeString(IXMLNode * xmlnode,AnsiString name)
{///Returns the string value for the node named name
	AnsiString ret = "0";
	_di_IXMLNodeList Nodes = xmlnode->ChildNodes;
	_di_IXMLNode node = Nodes->FindNode(name);
	if(node == NULL){																														///No node found
		AnsiString s = "XMLUtilities::GetXMLNodeString Node: "+name+" not found";
		Application->MessageBox(s.c_str(),"XML error",MB_OK);
		return ret;
	}
	try{																																				///Try to convert to value
		ret = node->Text;
	}
	catch(...){
		AnsiString s = "Utility::GetXMLNodeString Node: "+name+" cannot convert to double";
		Application->MessageBox(s.c_str(),"XML error",MB_OK);
	}
	return ret;
}
//===========================================================================
// XML Attribute functions
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::AddXMLNodeAttribute(IXMLNode * xmlnode,AnsiString att,AnsiString value)
{///Add attribute, att, and its value, value, to the node
	xmlnode->Attributes[att]=value;
}
//---------------------------------------------------------------------------
AnsiString  __fastcall XMLUtilities::XMLWrapAttr(AnsiString att,AnsiString value)
{///Create a string of the type: att="value"
	AnsiString s;
	s =" " + att+"="+"\""+value+"\"";
	return s;
}
//---------------------------------------------------------------------------
AnsiString  __fastcall XMLUtilities::GetXMLNodeAttribute(IXMLNode * xmlnode,AnsiString att)
{///Returns the value of the attribute att for named noded
	AnsiString ret = NULL;
	_di_IXMLNodeList Nodes = xmlnode->AttributeNodes;
	if(Nodes->Count == 0) return ret;
	_di_IXMLNode node = Nodes->FindNode(att);
	if(node == NULL){																														///No node found
		AnsiString s = "Utility::GetXMLNodeAttribute Node: "+att+" not found";
		Application->MessageBox(s.c_str(),"XML error",MB_OK);
		return ret;
	}
	try{																																				///Try to convert to value
		ret = node->Text;
	}
	catch(...){
		AnsiString s = "Utility::GetXMLNodeAttribute Node: "+att+" cannot convert to double";
		Application->MessageBox(s.c_str(),"XML error",MB_OK);
	}
	return ret;
}
//===========================================================================
/// XML file utilities
//---------------------------------------------------------------------------
bool __fastcall XMLUtilities::LoadXMLDocument(TXMLDocument *XMLDoc,AnsiString fn)
{///Load the XMLDocument from file fn
	bool ret = true;
	try{																																				///This will throw an error if the file is not well formed
		XMLDoc->Active   = false;
		XMLDoc->FileName = "";
		XMLDoc->XML->Clear();
		XMLDoc->LoadFromFile(fn);
		XMLDoc->Active   = true;
	}
	catch(...){																																	///Caught an error in trying to load the file
		AnsiString s ="Cannot parse "+fn+" as an XML Document";
		Application->MessageBox(s.c_str(),"Error",MB_OK);
		ret = false;
	}
	return ret;
}
//---------------------------------------------------------------------------
AnsiString  __fastcall XMLUtilities::XMLPath()
{///Returns the path to the XML directory
	AnsiString fp=ExtractFilePath(Application->ExeName);
	fp = fp+"XML\\";
	if(!DirectoryExists(fp))
		ForceDirectories(fp);
	return fp;
}
//===========================================================================
/// Base 64 encoder / decoder functions
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::Base64Encode(AnsiString slToEnc)
{//The Base64 Table
	const char Base64Table[64] =
		"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
	char * buftoenc = slToEnc.c_str();
	int bufsize = slToEnc.Length() + 1;
	char * encbuf = new char[slToEnc.Length() * 5];
	encbuf[0] = '\0';  int ilStrLen = -1;
	int i=0; int b64byte[5];
	unsigned char *buftemp;
	AnsiString slRetVal = EmptyStr;
	buftemp=(unsigned char *)malloc(bufsize+2);
	strcpy(buftemp,buftoenc);
	if (fmod(bufsize,3)==1)
		{
			buftemp[bufsize]='\0';
			buftemp[bufsize+1]='\0';
		}
	if (fmod(bufsize,3)==2)buftemp[bufsize]='\0';
	while (i<bufsize)
		{
			b64byte[0]=buftemp[i]>>2;
			b64byte[1]=((buftemp[i]&3)<<4)|(buftemp[i+1]>>4);
			b64byte[2]=((buftemp[i+1]&0x0F)<<2)|(buftemp[i+2]>>6);
			b64byte[3]=buftemp[i+2]&0x3F;
			encbuf[i+(i/3)]=   Base64Table[b64byte[0]];
			encbuf[i+(i/3)+1]= Base64Table[b64byte[1]];
			encbuf[i+(i/3)+2]= Base64Table[b64byte[2]];
			encbuf[i+(i/3)+3]= Base64Table[b64byte[3]];
			i+=3;
		}
	free(buftemp);
	if (fmod(bufsize,3)==0) ilStrLen = bufsize*8/6;
	else if (fmod(bufsize,3)==1) ilStrLen = ((bufsize+2)*8/6)-2;
	else if (fmod(bufsize,3)==2) ilStrLen = ((bufsize+1)*8/6)-1;
	else ilStrLen = -1;
	if(ilStrLen> 0) slRetVal = AnsiString(encbuf).SubString(1, ilStrLen);
	if(encbuf != NULL) { delete encbuf; encbuf = NULL; }
	return slRetVal;
}
//-----------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::Base64Decode(AnsiString slToDec)
{//Base64 Table
	const char Base64Table[64] =
		"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
	char * buftodec = slToDec.c_str();
	int bufsize = slToDec.Length() + 1;
	char * decbuf = new char[slToDec.Length() * 5];
	decbuf[0] = '\0';  int ilStrLen = -1;
	AnsiString slRetVal = EmptyStr;
	int i=0; unsigned char binbyte[4];
	int cpos[5]; unsigned char *buftemp;
	buftemp=(unsigned char *)malloc(bufsize);
	strcpy(buftemp,buftodec);
	if (fmod(bufsize,4)==1)
		{
			buftemp[bufsize]='\0';
			buftemp[bufsize+1]='\0';
			buftemp[bufsize+2]='\0';
		}
	if (fmod(bufsize,4)==2)
		{
			buftemp[bufsize]='\0';
			buftemp[bufsize+1]='\0';
		}
	if (fmod(bufsize,4)==3)buftemp[bufsize]='\0';
	while (i<bufsize)
		{
			cpos[0]=strchr(Base64Table,buftemp[i])-Base64Table;
			cpos[1]=strchr(Base64Table,buftemp[i+1])-Base64Table;
			cpos[2]=strchr(Base64Table,buftemp[i+2])-Base64Table;
			cpos[3]=strchr(Base64Table,buftemp[i+3])-Base64Table;
			binbyte[0]=((cpos[0]<<2)|(cpos[1]>>4));
			binbyte[1]=((cpos[1]<<4)|(cpos[2]>>2));
			binbyte[2]=(((cpos[2]&0x03)<<6)|(cpos[3]&0x3f));
			decbuf[i-(i/4)]=binbyte[0];
			decbuf[i-(i/4)+1]=binbyte[1];
			decbuf[i-(i/4)+2]=binbyte[2];
			i+=4;
		}
	free(buftemp);
	if (fmod(bufsize,4)==0) ilStrLen = bufsize*6/8;
	else if (fmod(bufsize,4)==1) ilStrLen = ((bufsize+3)*6/8)-3;
	else if (fmod(bufsize,4)==2) ilStrLen = ((bufsize+2)*6/8)-2;
	else if (fmod(bufsize,4)==3) ilStrLen = ((bufsize+1)*6/8)-1;
	else ilStrLen = -1;
	if(ilStrLen> 0) slRetVal = AnsiString(decbuf).SubString(1, ilStrLen);
	if(decbuf != NULL) { delete decbuf; decbuf = NULL; }
	return slRetVal;
}
//===========================================================================
/// XMLRPC header utilities
//---------------------------------------------------------------------------
bool __fastcall XMLUtilities::IsXMLRPC(AnsiString s)
{///Returns true if the header starts "POST /RPC" or "POST /RPC"
	if(s.Pos("POST /") != 0
		|| s.Pos("POST/") != 0
		|| s.Pos("HTTP") != 0) return true;
	return false;
}
//---------------------------------------------------------------------------
int	__fastcall XMLUtilities::FindXMLStart(AnsiString s)
{///Returns the starting position of <?xml
	int ret= s.Pos("<?xml");
	return ret;             															///Returns either the starting position of the xml, or 0 if no xml in file
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::GetUserAgent()
{///Returns the string required by HTTP/RPC requests
	AnsiString s;
	VerInfo * VersionData;
	VersionData = new VerInfo();
	AnsiString AOSVersion=VersionData->FixedFileVersion;                               ///Software version
	delete VersionData;
	s = "User-Agent: AOS/"+AOSVersion+" (WinXP)";
	return s;
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::GetHost()
{///Returns the string required by HTTP/RPC requests
	AnsiString s;
	s = "Host: "+
		GetServerSocketAddress()+":"+
		GetServerSocketPort();
	return s;
}
//---------------------------------------------------------------------------
int	__fastcall XMLUtilities::GetCharacterCount(TStrings * strngs)
{///Returns the number of characters in the strngs array
	int count = 0;
	for (int i=0; i < strngs->Count; i++)
		{///Loop over the array of strings
			AnsiString s = strngs->Strings[i];
			count = count + s.Length();
		}
	return count;
}
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::SetContentLength(TStrings * strngs)
{///Uses GetCharacterCount to determine # characters w/o the value being set, bumps the count by the # bytes, then sets the Content-Length: to the right value
	int num = GetCharacterCount(strngs);
	int ind = strngs->IndexOf("Content-length: "); ///Find the "Content-length: " string
	if(ind != -1)
		{///Found it
			AnsiString s = strngs->Strings[ind];
			AnsiString count = (AnsiString)num;								///Simple way to determine the number of digits
			num = num+count.Length();													///Make sure you have included the number of digits in the total length
			s = s+(AnsiString)num;
			strngs->Strings[ind] = s;													///Store the value
		}
	else
		{///The "Content-length: " string did not exist in the strngs object. This assumes that a header has been created
			strngs->Insert(2,"Content-length: ");
			SetContentLength(strngs);													 ///Call recursively
		}
}
//---------------------------------------------------------------------------
bool __fastcall XMLUtilities::XMLRPCStringToDOM(AnsiString str,TXMLDocument *XMLDoc)
{	/// Strip out all of the HTTP header stuff and load the XML into the document
	bool ret = true;
	if(!IsXMLRPC(str))
		return false;																				 	///This isn't an RPC string starting with POST
	int start = FindXMLStart(str);													///Find the first character of the XML string
	if(start == 0)
		return false;																				 	///There is no <?XML in this string
	AnsiString s = str.SubString(start,str.Length());
	try{
		XMLDoc->LoadFromXML(s);
	}
	catch(...){
		ret = false;
	}
	return ret;
}
//---------------------------------------------------------------------------
bool __fastcall XMLUtilities::ShowXMLRPCString(TXMLDocument * xmldoc,TTreeView * tree,AnsiString s)
{/// Convert an ansistring to an xmldoc, then show it in the tree
	bool ret = true;
	if(!XMLRPCStringToDOM(s,xmldoc)){												/// Convert the string to an xmldoc
		return false;																					/// Couldn't do it!
	}
	xmldoc->Active = true;
	XML2Tree(tree,xmldoc);																						/// Show what the XMLDoc looks like
	return ret;
}
//===========================================================================
/// Functions to traverse an XMLDoc
//---------------------------------------------------------------------------
_di_IXMLNode __fastcall XMLUtilities::ProcessNodeByName(IXMLNode * Node,AnsiString name)
{/// Recurse through nodes looking for node name
	if(Node == NULL){
		return NULL;					                                                    /// Can't process an empty node
	}
	IXMLNode * cNode;
	if(Node->GetNodeName() == name){
		return Node;																															/// Found it!
	}
	if(Node->ChildNodes->Count == 0){
	 return NULL;
	}
	cNode = Node->ChildNodes->First();
	IXMLNode * namenode = NULL;
	while(cNode != NULL){
		namenode = ProcessNodeByName(cNode, name);																/// See if this node is the one
		if(namenode != NULL){
			return namenode;																												/// Found it!
		}
		cNode= cNode->NextSibling();
	}
	return NULL;																																/// Wasn't found
}
//---------------------------------------------------------------------------
_di_IXMLNode __fastcall XMLUtilities::GetNodeByName(TXMLDocument *XMLDoc,AnsiString name)
{/// Return the node called name in document xmldoc
	IXMLNode * jNode;
	IXMLNode * namenode = NULL;
	XMLDoc->Active=true;                                                        /// Make sure the XMLDocument is active... parses the XML
	if(XMLDoc->ChildNodes->First() == NULL){
		return NULL;																                             	/// No nodes!
	}
	jNode = XMLDoc->ChildNodes->Nodes[0];                                       /// Start at the first node
	while(jNode != NULL){																												/// Walk down through the siblings
		namenode = ProcessNodeByName(jNode,name);
		if(namenode != NULL){																											/// Found the node!
			return namenode;
		}
		jNode = jNode->NextSibling();
	}
	return NULL;																																/// Couldn't find it
}
//---------------------------------------------------------------------------
bool  __fastcall XMLUtilities::GetMethodName(TXMLDocument * XMLDoc,AnsiString *val)
{///	Return the methodName string <methodName>examples.getStateName</methodName>
	bool ret = true;
	_di_IXMLNode node = GetNodeByName(XMLDoc,"methodName");											/// Get the node named methodName
	if(node == NULL){																														/// Will return an empty string
		return false;
	}
	try{
		if(node->IsTextElement)                                                     ///This node has only a value, no child nodes
			*val = (AnsiString)node->Text;
		else ret = false;
	}
	catch(...){
		ret = false;
	}
	return ret;
}
//---------------------------------------------------------------------------
bool __fastcall XMLUtilities::GetXMLRPCValueNode(IXMLNode * xmlnode)
{///Returns the node as <type>val</type>
	bool ret = true;
	_di_IXMLNodeList Nodes = xmlnode->ChildNodes;
	_di_IXMLNode Node = Nodes->FindNode("value");
	if(Node == NULL || Node->ChildNodes->Count == 0)														///No node found
		return false;
	xmlnode = Node->ChildNodes->First();                                        ///Set this to the <type>val</val> node
	return ret;
}
//---------------------------------------------------------------------------
bool __fastcall XMLUtilities::CheckTypeString(AnsiString type)
{///Returns true if the type string is covered in XMLRPC spec
	bool ret = true;
	if(TypeStrings->IndexOf(type) < 0)										///Index will be >=0 if the string is found
		ret = false;
	return ret;
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::GetBoolString(bool value)
{///Returns 1 or 0 for true/false
	AnsiString s ="1";
	if(!value) s = "0";
	return s;
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::GetTimeString(TDateTime value)
{///Returns ISO8601 time string for XMLRPC
	AnsiString s;
	AnsiString year = (AnsiString)YearOf(value);
	AnsiString month = (AnsiString)MonthOf(value);
	if(month.Length() ==1)
		month = "0"+month;
	AnsiString day = (AnsiString)DayOf(value);
	day = day+"T";																													///Add separator
	AnsiString hour = HourOf(value);
	AnsiString min  = MinuteOf(value);
	AnsiString sec  = SecondOf(value);
	s = year+month+day+hour+min+sec;
	return s;
}
//===========================================================================
///XMLRPC encoding & wrappers
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCWrap(double value)
{///Wrap a double as <double>value</double>
	AnsiString s;
	s = "<double>"+(AnsiString)value+"</double>";
	return s;
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCWrap(int value)
{//Wrap an integer as <int>value</int>
	AnsiString s;
	s = "<int>"+(AnsiString)value+"</int>";
	return s;
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCWrap(AnsiString value)
{///Wrap a string as <string>value</string>
	AnsiString s;
	s = "<string>"+(AnsiString)value+"</string>";
	return s;
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCWrap(bool value)
{///Wrap a bool as <bool>value</bool>, where value will be 1 = true or 0 = false
	AnsiString s;
	s = "<boolean>"+GetBoolString(value)+"</boolean>";
	return s;
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCWrap(TDateTime value)
{///Wrap a TDateTime as <dateTime.iso8601>19980717T14:08:55</dateTime.iso8601>
	AnsiString s;
	s = "<dateTime.iso8601>"+GetTimeString(value)+"</dateTime.iso8601>";     ///NOTE the timezone is not implemented in RPC
	return s;
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCWrapAsBinary(AnsiString value)
{///Encode the AnsiString as Base64, wrap as <base64>ABCDabcd1234</base64>
	AnsiString s;
	AnsiString b64 = Base64Encode(value);
	s = "<base64>"+b64+"</base64>";
	return s;
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCWrapValue(AnsiString value)
{///Returns <value>value</value>
	AnsiString s;
	s = "<value>";
	s = s + (AnsiString)value;
	s = s + "</value>";
	return s;
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCWrapMember(AnsiString name,AnsiString value)
{///Returns  <member><name>name</name><value><i4>number</i4></value></member>
	AnsiString s;
	s = "<member>";
	s = s + "<name>"+name+"</name>";
	s = s + XMLRPCWrapValue(value);
	s = s + "</member>";
	return s;
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCWrapParam(AnsiString value)
{///Returns <param><type> value </type></param>, value must be pre-wrapped
	AnsiString s;
	s = "<param>";
	s = s + value;
	s = s + "</param>";
	return s;
}
//===========================================================================
/// XMLRPC array functions
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCStartArray(){return "<array><data>";}						///Returns <array><data>
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCEndArray(){	return "</data></array>";}					///Returns </data></array>
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCArrayValue(AnsiString value)
{///Returns <value><type>value</type></value> based on the type
	AnsiString s = XMLRPCWrap(value);																	 ///Use overloaded wrapper to get string
	return XMLRPCWrapValue(s);
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCArrayValue(double value)
{///Returns <value><type>value</type></value> based on the type
	AnsiString s = XMLRPCWrap(value);																	 ///Use overloaded wrapper to get string
	return XMLRPCWrapValue(s);
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCArrayValue(int value)
{///Returns <value><type>value</type></value> based on the type
	AnsiString s = XMLRPCWrap(value);																	 ///Use overloaded wrapper to get string
	return XMLRPCWrapValue(s);
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCArrayValue(bool value)
{///Returns <value><type>value</type></value> based on the type
	AnsiString s = XMLRPCWrap(value);																	 ///Use overloaded wrapper to get string
	return XMLRPCWrapValue(s);
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCArrayValue(TDateTime value)
{///Returns <value><type>value</type></value> based on the type
	AnsiString s = XMLRPCWrap(value);																	 ///Use overloaded wrapper to get string
	return XMLRPCWrapValue(s);
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCBinaryArrayValue(AnsiString value)
{///Returns <value><type>value</type></value> based on the type
	AnsiString s = XMLRPCWrapAsBinary(value);													 ///Use overloaded wrapper to get string
	return XMLRPCWrapValue(s);
}
//===========================================================================
/// XMLRPC struct functions
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCStartStruct(){return "<struct>";}		///Returns <struct>
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCEndStruct(){return "</struct>";}		///Returns </struct>
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCStructMember(AnsiString name,AnsiString value)
{///Returns a fully wrapped structure value
	AnsiString s = XMLRPCWrap(value);																	 ///Use overloaded wrapper to get string
	return XMLRPCWrapMember(name,s);
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCStructMember(AnsiString name,double value)
{///Returns a fully wrapped structure value
	AnsiString s = XMLRPCWrap(value);																	 ///Use overloaded wrapper to get string
	return XMLRPCWrapMember(name,s);
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCStructMember(AnsiString name,int value)
{///Returns a fully wrapped structure value
	AnsiString s = XMLRPCWrap(value);																	 ///Use overloaded wrapper to get string
	return XMLRPCWrapMember(name,s);
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCStructMember(AnsiString name,bool value)
{///Returns a fully wrapped structure value
	AnsiString s = XMLRPCWrap(value);																	 ///Use overloaded wrapper to get string
	return XMLRPCWrapMember(name,s);
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCStructMember(AnsiString name,TDateTime value)
{///Returns a fully wrapped structure value
	AnsiString s = XMLRPCWrap(value);																	 ///Use overloaded wrapper to get string
	return XMLRPCWrapMember(name,s);
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCBinaryStructMember(AnsiString name,AnsiString value)
{///Returns a fully wrapped structure value
	AnsiString s = XMLRPCWrapAsBinary(value);																	 ///Use overloaded wrapper to get string
	return XMLRPCWrapMember(name,s);
}
//===========================================================================
///XMLRPC Parameter wrappers
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCStartParams(){return "<params>";}					///Returns the start of the method params
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCEndParams(){return "</params>";}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCParamValue(AnsiString value)
{///Returns <param><value><type>value</type></value></param> based on the type
	AnsiString s = XMLRPCWrap(value);																	 ///Use overloaded wrapper to get string
	return XMLRPCWrapParam(s);
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCParamValue(double value)
{///Returns <param><value><type>value</type></value></param> based on the type
	AnsiString s = XMLRPCWrap(value);																	 ///Use overloaded wrapper to get string
	return XMLRPCWrapParam(s);
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCParamValue(int value)
{///Returns <param><value><type>value</type></value></param> based on the type
	AnsiString s = XMLRPCWrap(value);																	 ///Use overloaded wrapper to get string
	return XMLRPCWrapParam(s);
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCParamValue(bool value)
{///Returns <param><value><type>value</type></value></param> based on the type
	AnsiString s = XMLRPCWrap(value);																	 ///Use overloaded wrapper to get string
	return XMLRPCWrapParam(s);
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCParamValue(TDateTime value)
{///Returns <param><value><type>value</type></value></param> based on the type
	AnsiString s = XMLRPCWrap(value);																	 ///Use overloaded wrapper to get string
	return XMLRPCWrapParam(s);
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCBinaryParamValue(AnsiString value)
{///Returns <param><value><type>value</type></value></param> based on the type
	AnsiString s = XMLRPCWrapAsBinary(value);													///Use overloaded wrapper to get string
	return XMLRPCWrapParam(s);
}
//===========================================================================
///XMLRPC Request Method functions
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::CreateXMLRPCRequestHeader(TStrings * strngs)
{///Creates the XMLRPC request header portion of the header of the sort:
	/*POST /RPC2 HTTP/1.0
		User-Agent: Frontier/5.1.2 (WinNT)
		Host: betty.userland.com
		Content-Type: text/xml
		Content-length: 181	*/
	strngs->Add("POST /RPC HTTP/1.0");
	strngs->Add(GetUserAgent());
	strngs->Add(GetHost());
	strngs->Add("Content-Type: text/xml");
	strngs->Add("Content-length: ");
	strngs->Add(XMLDeclaration());
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCStartMethodCall(AnsiString method)
{///Returns <methodCall><methodName>method</methodName>
	return "<methodCall><methodName>"+method+"</methodName>";
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCEndMethodCall(){return "</methodCall>";}																												///End the method call wrapper
//===========================================================================
/// XMLRPC Response functions
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::CreateXMLRPCResponseHeader(TStrings * strngs)
{///Creates the XMLRPC header portion of the header of the sort:
/* should look like this-
HTTP/1.1 200 OK
Connection: Keep-Alive
Content-Length: 158
Content-Type: text/xml
Date: Fri, 17 Jul 1998 19:55:08 GMT
Server: UserLand Frontier/5.1.2-WinNT*/
	AnsiString s;
	strngs->Add("HTTP/1.1 200 OK");
	strngs->Add("Connection: Keep-Alive");
	strngs->Add("Content-length: ");
	strngs->Add("Content-Type: text/xml");
	strngs->Add("Date: " + Now());
	strngs->Add("Server: "+ GetServerSocketAddress());
	strngs->Add(XMLDeclaration());
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCStartMethodResponse()
{///Returns the starting block for a method response
	AnsiString s = "<methodResponse>";
	//s = s + XMLRPCStartParams();
	s = s + "<params>";
	s = s + "<param>";																													/// Start the response using the <param >
	s = s+ "<value>";																														/// Start the response <value>
	return s;
}
//---------------------------------------------------------------------------
AnsiString __fastcall XMLUtilities::XMLRPCEndMethodResponse()
{///Returns the end of a method response block
	//AnsiString s = XMLRPCEndParams();
	AnsiString s = "</value>";																									/// Start the response <value>
	s = s + "</param>";
	s = s + "</params>";																												/// Start the response using the <param >
	s = s +"</methodResponse>";
	return s;
}
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::StartAOSXMLRPCMethodResponse(AnsiString method,TStrings * strngs)
{/// Puts together the AOS version of an XMLRPC method response
/*
<methodResponse>
 <params>
<param> <value><struct>
	<member>
				<name>Procecure</name>
				<value><string>method</string></value>
	</member>
*/
	CreateXMLRPCResponseHeader(strngs);																					/// Create the response header
	strngs->Add(XMLRPCStartMethodResponse());																		/// Add the <methodResponse>"+XMLRPCStartParams() strings
	strngs->Add(XMLRPCStartStruct());																						/// This response contains a structure
	strngs->Add(XMLRPCStructMember("Procedure",method));
}
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::EndAOSXMLRPCMethodResponse(TStrings * strngs)
{/// Complete the AOS XMLRPC Method response
	strngs->Add(XMLRPCEndStruct());																		/// This response contains a structure
	strngs->Add(XMLRPCEndMethodResponse());													/// Add the XMLRPCEndParams()+</methodResponse> strings
	SetContentLength(strngs);
}
//===========================================================================
///XMLRPC Fault response
//---------------------------------------------------------------------------
void __fastcall XMLUtilities::XMLRPCFaultResponse(int code,AnsiString fault,TStrings * strngs)
{///Returns the response
	/*<methodResponse>
		<fault>
			<value>
				<struct>
					<member>
						<name>faultCode</name>
						<value><int>4</int></value>
					</member>
					<member>
						<name>faultString</name>
						<value><string>Too many parameters.</string></value>
					</member>
				</struct>
			</value>
		</fault>
	</methodResponse>*/
 strngs->Add("<methodResponse>");
 strngs->Add("<fault>");
 strngs->Add("<value>");
 strngs->Add(XMLRPCStartStruct());
 strngs->Add(XMLRPCStructMember("faultCode",code));
 strngs->Add(XMLRPCStructMember("faultString",fault));
 strngs->Add(XMLRPCEndStruct());
 strngs->Add("</value>");
 strngs->Add("</fault>");
 strngs->Add("</methodResponse>");
}
//---------------------------------------------------------------------------
//===========================================================================
///Registry functions
//---------------------------------------------------------------------------
TRegistry * XMLUtilities::GetRegistry()
{///Utility function to create the registry key values and open the key. Deletes registry and sets it NULL if unsuccessful
	AnsiString KeyName;
	TRegistry *Registry = new TRegistry(KEY_READ);
	Registry->RootKey=HKEY_LOCAL_MACHINE;           /// Keep AUC-AOS information here so globally available
	KeyName="SOFTWARE\\CAMIS\\AUC\\XL\\Config";     /// Registry key where configuration parameters kept.
	Registry->Access=KEY_ALL_ACCESS;                /// Need to make sure that any user logged into computer (not software) has access
	try
		{
			Registry->OpenKey(KeyName,true);            /// False because we do not want to create it if it doesn't exist
		}
	catch(...)
		{
			if(Registry) delete Registry;
			Registry = NULL;
		}
	return Registry;
}
//===========================================================================
///Server socket
//---------------------------------------------------------------------------
AnsiString XMLUtilities::GetServerSocketAddress()
{///Returns the ServerSocket address
	AnsiString S="127.0.0.1";
	TRegistry *Registry = GetRegistry();
	if(Registry)
		{
			if(Registry->ValueExists("ServerSocketAddress"))
				S = Registry->ReadString("ServerSocketAddress");
			else SetServerSocketAddress(S);
			Registry->CloseKey();
			delete Registry;
		}
	return S;
}
//---------------------------------------------------------------------------
bool XMLUtilities::SetServerSocketAddress(AnsiString msg)
{///Save the Server socket address in the registry
	bool ret=true;
	TRegistry *Registry = GetRegistry();
	if(Registry)
		{///Try to register
			Registry->WriteString("ServerSocketAddress",msg);
			Registry->CloseKey();
			delete Registry;
		}
	else ret=false;
	return ret;
}
//---------------------------------------------------------------------------
AnsiString XMLUtilities::GetServerSocketPort()
{///Returns the ServerSocket port
	AnsiString S="10000";
	TRegistry *Registry = GetRegistry();
	if(Registry)
		{
			if(Registry->ValueExists("ServerSocketPort"))
				S = Registry->ReadString("ServerSocketPort");
			else SetServerSocketPort(S);
			Registry->CloseKey();
			delete Registry;
		}
	return S;
}
//---------------------------------------------------------------------------
bool XMLUtilities::SetServerSocketPort(AnsiString msg)
{///Save the server socket port in the registry
	bool ret=true;
	TRegistry *Registry = GetRegistry();
	if(Registry)
		{///Try to register
			Registry->WriteString("ServerSocketPort",msg);
			Registry->CloseKey();
			delete Registry;
		}
	else ret=false;
	return ret;
}
//===========================================================================
///Client socket
//---------------------------------------------------------------------------
AnsiString XMLUtilities::GetClientSocketAddress()
{///Returns the ServerSocket address
	AnsiString S="127.0.0.1";
	TRegistry *Registry = GetRegistry();
	if(Registry)
		{
			if(Registry->ValueExists("ClientSocketAddress"))
				S = Registry->ReadString("ClientSocketAddress");
			else SetClientSocketAddress(S);
			Registry->CloseKey();
			delete Registry;
		}
	return S;
}
//---------------------------------------------------------------------------
bool XMLUtilities::SetClientSocketAddress(AnsiString msg)
{///Save the Server socket address in the registry
	bool ret=true;
	TRegistry *Registry = GetRegistry();
	if(Registry)
		{///Try to register
			Registry->WriteString("ClientSocketAddress",msg);
			Registry->CloseKey();
			delete Registry;
		}
	else ret = false;
	return ret;
}
//---------------------------------------------------------------------------
AnsiString XMLUtilities::GetClientSocketPort()
{///Returns the ServerSocket port
	AnsiString S="10001";
	TRegistry *Registry = GetRegistry();
	if(Registry)
		{
			if(Registry->ValueExists("ClientSocketPort"))
				S = Registry->ReadString("ClientSocketPort");
			else SetClientSocketPort(S);
			Registry->CloseKey();
			delete Registry;
		}
	return S;
}
//---------------------------------------------------------------------------
bool XMLUtilities::SetClientSocketPort(AnsiString msg)
{///Save the server socket port in the registry
	bool ret=true;
	TRegistry *Registry = GetRegistry();
	if(Registry)
		{///Try to register
			Registry->WriteString("ClientSocketPort",msg);
			Registry->CloseKey();
			delete Registry;
		}
	else ret = false;
	return ret;
}
//---------------------------------------------------------------------------

#pragma package(smart_init)
