//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#ifndef XMLUtilitiesH
#define XMLUtilitiesH
#include <msxmldom.hpp>
#include <XMLDoc.hpp>
#include <xmldom.hpp>
#include <XMLIntf.hpp>
#include <ComCtrls.hpp>
#include <Registry.hpp>
class XMLUtilities:public TComponent
{
private:
	void __fastcall ProcessNode(IXMLNode * Node,TTreeView * tree,TTreeNode * tn);        	///Copies info from the XML node to the Tree Node
	void __fastcall ProcessTreeItem(IXMLNode * Node,TTreeView * tree,TTreeNode * tn);    	///Copies info from the Tree node to the XML Node
	AnsiString  __fastcall GetUserAgent();																							 	///Returns the string required by HTTP/RPC requests
	AnsiString  __fastcall GetHost();																											///Returns the string required by HTTP/RPC requests
	AnsiString  __fastcall Base64Encode(AnsiString slToEnc);															///Encode a string to a Base64 string
	AnsiString  __fastcall Base64Decode(AnsiString slToDec);															///Decode a string into a Base64 string
	TStringList *  TypeStrings;		  																											///Strings used to test whether a data type is valid
	AnsiString  __fastcall GetBoolString(bool value);																			///Returns 1 or 0 for true/false
	AnsiString  __fastcall GetTimeString(TDateTime value);																///Returns ISO8601 time string for XMLRPC
 ///Server socket
	AnsiString  GetServerSocketAddress();                 															 	///Returns the ServerSocket address
	bool        SetServerSocketAddress(AnsiString msg);   															 	///Save the Server socket address in the registry
	AnsiString  GetServerSocketPort();                    															 	///Returns the ServerSocket port
	bool        SetServerSocketPort(AnsiString msg);      															 	///Save the server socket port in the registry
 ///Client socket
	AnsiString  GetClientSocketAddress();                	 															 	///Returns the ServerSocket address
	bool        SetClientSocketAddress(AnsiString msg);   															 	///Save the Server socket address in the registry
	AnsiString  GetClientSocketPort();                    															 	///Returns the ServerSocket port
	bool        SetClientSocketPort(AnsiString msg);      															 	///Save the server socket port in the registry
public:
		__fastcall XMLUtilities(TComponent *Owner);
		__fastcall ~XMLUtilities();
 ///Utilities
	AnsiString					__fastcall NewGUIDString();																				/// Create a unique GUID
	System::TGUID				__fastcall NewGUID();																							/// Returns a GUID
	System::TGUID				__fastcall StringToGUID(AnsiString s);														/// Returns a GUID
	AnsiString					__fastcall GUIDToString(System::TGUID uuid);                    	/// Returns a String representation of the UUID
	AnsiString  				__fastcall StringList2String(TStrings * strngs);									/// Convert a stringlist to a single string
 /// Functions to traverse an XMLDoc
	_di_IXMLNode __fastcall ProcessNodeByName(IXMLNode * Node,AnsiString name);        		/// Recurse through nodes looking for node name
	_di_IXMLNode __fastcall GetNodeByName(TXMLDocument *XMLDoc,AnsiString name);					/// Return the node called name in document xmldoc
 ///Registry functions
	TRegistry * GetRegistry();                                                  					///Utility function to create the registry key values and open the key. Deletes registry and sets it NULL if unsuccessful
 ///XML utilities
	void        __fastcall CheckXMLDeclaration(TStrings * strngs);              	///Checks to see if there is a standard XML declaration, and adds one if not
	bool        __fastcall HasXMLDeclaration(TStrings * strngs);                	///Check to see if the strngs has a valid (for AOS) declaration
	bool        __fastcall OnlyOneXMLDeclaration(TStrings * strngs);            	///Check to see if the strngs only one valid declaration
	void        __fastcall RemoveAllXMLDeclarations(TStrings * strngs);         	///Remove all XML declarations
	void        __fastcall FixXMLDeclaration(TStrings * strngs);                	///Make sure there is only one declaration and that it is the first element of strngs
	void        __fastcall AddXMLDeclaration(TStrings * strngs);                	///Adds a standard XML declaration as the first string
	AnsiString  __fastcall XMLDeclaration()
		{return "<?xml version='1.0' encoding='UTF-8'?>";}                        	///Inserts a standard XML declaration
	AnsiString  __fastcall XMLWrap(AnsiString name,double value);               	///Wrap a double as <name>value</name>,
	AnsiString  __fastcall XMLWrap(AnsiString name,int value);                  	///Wrap an integer as <name>value</name>,
	AnsiString  __fastcall XMLWrap(AnsiString name,AnsiString value);           	///Wrap a string as <name>value</name>,
	AnsiString  __fastcall XMLWrap(AnsiString name,bool value);                 	///Wrap a bool as <name>value</name>, where value will be 1 = true or 0 = false
	AnsiString  __fastcall XMLWrap(AnsiString name,AnsiString attr,AnsiString value); ///Add atributes to name, then wrap it with the string value
	AnsiString  __fastcall XMLWrapAttr(AnsiString att,AnsiString value);        	///Create a string of the type: att="value"
	AnsiString  __fastcall StartXMLNode(AnsiString name){return "<"+name+">";}  	///Wrap name as <name>
	AnsiString  __fastcall EndXMLNode(AnsiString name){return "</"+name+">";}   	///Wrap name as </name>
	void        __fastcall XML2Tree(TTreeView * treeview,TXMLDocument *XMLDoc); 	///Create a tree view of an XML document
	void        __fastcall Tree2XML(TTreeView * treeview,TXMLDocument *XMLDoc); 	///Create a tree view of an XML document
	void				__fastcall NodeExpandAll(TTreeNode * node);												///Recursively expand all child nodes of the node that is passed in
	void				__fastcall TreeExpandAll(TTreeView * node);												///Recursively expand all child nodes of the node that is passed in
	void				__fastcall NodeCollapseAll(TTreeNode * node);											///Recursively collapse all child nodes of the node that is passed in
	void				__fastcall TreeCollapseAll(TTreeView * node);											///Recursively collapse all child nodes of the node that is passed in
	int         __fastcall GetXMLNodeInteger(IXMLNode * xmlnode,AnsiString name); ///Returns the integer value for the node named name
	double      __fastcall GetXMLNodeDouble(IXMLNode * xmlnode,AnsiString name);  ///Returns the double value for the node named name
	AnsiString  __fastcall GetXMLNodeString(IXMLNode * xmlnode,AnsiString name);  ///Returns the double value for the node named name
	AnsiString  __fastcall GetXMLNodeAttribute(IXMLNode * xmlnode,AnsiString att);///Returns the value of the attribute att for named node
	void        __fastcall AddXMLNodeAttribute(IXMLNode * xmlnode,AnsiString att,AnsiString value); ///Add attribute, att, and its value, value, to the node
	bool        __fastcall LoadXMLDocument(TXMLDocument *XMLDoc,AnsiString fn);   ///Load the XMLDocument from file fn
	AnsiString  __fastcall XMLPath();                                             ///Returns the path to the XML directory
	//XMLRPC utilities
	int					__fastcall FindXMLStart(AnsiString s);														///Returns the starting position of <?xml
	bool				__fastcall IsXMLRPC(AnsiString s);																///Returns true if the header starts "POST /RPC"
	int					__fastcall GetCharacterCount(TStrings * strngs);									///Returns the number of characters in the strngs array
	void				__fastcall SetContentLength(TStrings * strngs);										///Uses GetCharacterCount to determine # characters w/o the value being set, bumps the count by the # bytes, then sets the Content-Length: to the right value
	bool				__fastcall CheckTypeString(AnsiString type);											///Returns true if the type string is covered in XMLRPC spec
	bool				__fastcall XMLRPCStringToDOM(AnsiString str,TXMLDocument *XMLDoc);/// Strip out all of the HTTP header stuff and load the XML into the document
	bool				__fastcall ShowXMLRPCString(TXMLDocument *xmldoc, TTreeView * tree,AnsiString s);				/// Convert an ansistring to an xmldoc, then show it in the tree
	bool  			__fastcall GetMethodName(TXMLDocument * XMLDoc,AnsiString *val);	///	Get methodName string <methodName>examples.getStateName</methodName> in val, return true if successful
 ///XMLRPC- get the value node
	bool __fastcall GetXMLRPCValueNode(IXMLNode * xmlnode);												/// Sets xmlnode to the <type>val</type>, return true if successful
 ///XMLRPC encoding & wrappers
	AnsiString  __fastcall XMLRPCWrap(double value);              								///Wrap a double as <double>value</double>
	AnsiString  __fastcall XMLRPCWrap(int value);                 								///Wrap an integer as <int>value</int>
	AnsiString  __fastcall XMLRPCWrap(AnsiString value);          								///Wrap a string as <string>value</string>
	AnsiString  __fastcall XMLRPCWrap(bool value);                								///Wrap a bool as <bool>value</bool>, where value will be 1 = true or 0 = false
	AnsiString  __fastcall XMLRPCWrap(TDateTime value);														///Wrap a TDateTime as <dateTime.iso8601>19980717T14:08:55</dateTime.iso8601>
	AnsiString  __fastcall XMLRPCWrapAsBinary(AnsiString value);									///Encode the AnsiString as Base64, wrap as <base64>ABCDabcd1234</base64>
	AnsiString  __fastcall XMLRPCWrapValue(AnsiString value); 										///Returns <value>value</value>
	AnsiString  __fastcall XMLRPCWrapMember(AnsiString name,AnsiString value);		///Returns a fully wrapped structure value
	AnsiString  __fastcall XMLRPCWrapParam(AnsiString value);											///Returns <param><type>value</type></param>
 ///XMLRPC array functions
	AnsiString  __fastcall XMLRPCStartArray();																		///Returns <array><data>
	AnsiString  __fastcall XMLRPCEndArray();																			///Returns </data></array>
	AnsiString	__fastcall XMLRPCArrayValue(AnsiString value);										///Returns <value><type>value</type></value> based on the type
	AnsiString	__fastcall XMLRPCArrayValue(double value);												///Returns <value><type>value</type></value> based on the type
	AnsiString	__fastcall XMLRPCArrayValue(int value);														///Returns <value><type>value</type></value> based on the type
	AnsiString	__fastcall XMLRPCArrayValue(bool value);													///Returns <value><type>value</type></value> based on the type
	AnsiString	__fastcall XMLRPCArrayValue(TDateTime value);											///Returns <value><type>value</type></value> based on the type
	AnsiString	__fastcall XMLRPCBinaryArrayValue(AnsiString value);							///Returns <value><type>value</type></value> based on the type
 ///XMLRPC struct functions
	AnsiString	__fastcall XMLRPCStartStruct();																		///Returns <struct>
	AnsiString	__fastcall XMLRPCEndStruct();																			///Returns </struct>
	AnsiString  __fastcall XMLRPCStructMember(AnsiString name,AnsiString value);	///Returns a fully wrapped structure value
	AnsiString  __fastcall XMLRPCStructMember(AnsiString name,double value);			///Returns a fully wrapped structure value
	AnsiString  __fastcall XMLRPCStructMember(AnsiString name,int value);					///Returns a fully wrapped structure value
	AnsiString  __fastcall XMLRPCStructMember(AnsiString name,bool value);				///Returns a fully wrapped structure value
	AnsiString  __fastcall XMLRPCStructMember(AnsiString name,TDateTime value);		///Returns a fully wrapped structure value
	AnsiString  __fastcall XMLRPCBinaryStructMember(AnsiString name,AnsiString value);	///Returns a fully wrapped structure value
 ///XMLRPC Parameter wrappers
	AnsiString  __fastcall XMLRPCStartParams();																		///Returns the start of the method params
	AnsiString  __fastcall XMLRPCEndParams();																			///Returns </params>
	AnsiString	__fastcall XMLRPCParamValue(AnsiString value);										///Returns <param><value><type>value</type></value></param> based on the type
	AnsiString	__fastcall XMLRPCParamValue(double value);												///Returns <param><value><type>value</type></value></param> based on the type
	AnsiString	__fastcall XMLRPCParamValue(int value);														///Returns <param><value><type>value</type></value></param> based on the type
	AnsiString	__fastcall XMLRPCParamValue(bool value);													///Returns <param><value><type>value</type></value></param> based on the type
	AnsiString	__fastcall XMLRPCParamValue(TDateTime value);											///Returns <param><value><type>value</type></value></param> based on the type
	AnsiString	__fastcall XMLRPCBinaryParamValue(AnsiString value);							///Returns <param><value><type>value</type></value></param> based on the type
 ///XMLRPC Request Method call
	void				__fastcall CreateXMLRPCRequestHeader(TStrings * strngs); 					///Creates the XMLRPC header portion of the header
	AnsiString	__fastcall XMLRPCStartMethodCall(AnsiString method);							///Returns <methodCall><methodName>method</methodName>
	AnsiString  __fastcall XMLRPCEndMethodCall();																	///End the method call wrapper
 ///XMLRPC Response
	void				__fastcall CreateXMLRPCResponseHeader(TStrings * strngs);					///Creates a method response header
	AnsiString  __fastcall XMLRPCStartMethodResponse();														///Returns the starting block for a method response
	AnsiString  __fastcall XMLRPCEndMethodResponse(); 														///Returns the end of a method response block
	void 				__fastcall StartAOSXMLRPCMethodResponse(AnsiString method,TStrings * strngs);	/// Returs the AOS version of an XMLRPC Method Response method is the Machine.xxx string needing a response
	void 				__fastcall EndAOSXMLRPCMethodResponse(TStrings * strngs);					/// Complete the AOS XMLRPC Method response
 ///XMLRPC Fault response
	void  			__fastcall XMLRPCFaultResponse(int code,AnsiString fault,TStrings * strngs);  ///Returns the response
};
extern XMLUtilities *XMLUtil;
//---------------------------------------------------------------------------
#endif
